@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,17 +16,11 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6904 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-02-26 16:26:03 +0100 (Di, 26. Feb 2013) $
-
 ********************************************************************/
 
 
 #ifndef FIRSTTIMEHELPDIALOG_H
-#define FIRSTTIMEHELPDialog_H
+#define FIRSTTIMEHELPDIALOG_H
 
 #include <QDialog>
 #include <QLabel>
@@ -41,16 +35,16 @@ private:
 	~FirstTimeHelpDialog();
 
 protected:
-    static void init();
+	static void init();
 
 public:
-    static void setViewID(ViewLayer::ViewID);
-    static void showFirstTimeHelp();
-    static void hideFirstTimeHelp();
-    static void cleanup();
+	static void setViewID(ViewLayer::ViewID);
+	static void showFirstTimeHelp();
+	static void hideFirstTimeHelp();
+	static void cleanup();
 
 protected:
-    QLabel * m_label;
+	QLabel * m_label;
 };
 
 #endif
