@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,12 +16,6 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public Licensetriple
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6998 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
-
 ********************************************************************/
 
 #ifndef ITEMBASE_H
@@ -33,7 +27,12 @@ $Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 #include <QHash>
 #include <QList>
 #include <QGraphicsSceneHoverEvent>
+#include <QtGlobal>
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QGraphicsSvgItem>
+#else
+#include <QtSvgWidgets/QGraphicsSvgItem>
+#endif
 #include <QPointer>
 #include <QUrl>
 #include <QMap>
@@ -45,14 +44,30 @@ $Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 #include "../utils/misc.h"
 
 class ConnectorItem;
+class ModelPart;
+class FSvgRenderer;
+class ModelPartShared;
+class Bus;
+class Wire;
+class PartLabel;
+class LayerAttributes;
+class Connector;
+class ReferenceModel;
 
-typedef QMultiHash<ConnectorItem *, ConnectorItem *> ConnectorPairHash;
-
-typedef bool (*SkipCheckFunction)(ConnectorItem *);
-
+using ConnectorPairHash = QMultiHash<ConnectorItem*, ConnectorItem*>;
+using SkipCheckFunction = bool(ConnectorItem*);
+class ModelPartShared;
+class ModelPart;
+class Bus;
+class Wire;
+class PartLabel;
+class FSvgRenderer;
+class LayerAttributes;
+class Connector;
+class ReferenceModel;
 class ItemBase : public QGraphicsSvgItem
 {
-Q_OBJECT
+	Q_OBJECT
 
 public:
 	enum PluralType {
@@ -62,30 +77,31 @@ public:
 	};
 
 public:
-	ItemBase(class ModelPart*, ViewLayer::ViewID, const ViewGeometry &, long id, QMenu * itemMenu);
+	explicit ItemBase(ModelPart*, ViewLayer::ViewID, const ViewGeometry &, long id, QMenu * itemMenu);
 	virtual ~ItemBase();
 
-	qint64 id() const;
+	constexpr qint64 id() const noexcept { return m_id; }
 	void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
 	double z();
 	virtual void saveGeometry() = 0;
 	ViewGeometry & getViewGeometry();
 	ViewGeometry::WireFlags wireFlags() const;
 	virtual bool itemMoved() = 0;
-	QSizeF size();
-	class ModelPart * modelPart();
-	void setModelPart(class ModelPart *);
-	class ModelPartShared * modelPartShared();
+	constexpr QSizeF size() const noexcept { return m_size; }
+	ModelPart * modelPart();
+	void setModelPart(ModelPart *);
+	ModelPartShared * modelPartShared();
 	virtual void writeXml(QXmlStreamWriter &) {}
-	virtual void saveInstance(QXmlStreamWriter &);
+	virtual void saveInstance(QXmlStreamWriter &, bool flipAware);
 	virtual void saveInstanceLocation(QXmlStreamWriter &) = 0;
 	virtual void writeGeometry(QXmlStreamWriter &);
 	virtual void moveItem(ViewGeometry &) = 0;
-	virtual void setItemPos(QPointF & pos);
+	virtual void setItemPos(const QPointF & pos);
+	virtual void setLocation(const QPointF & loc);
 	virtual void rotateItem(double degrees, bool includeRatsnest);
 	virtual void flipItem(Qt::Orientations orientation);
 	virtual void transformItem(const QTransform &, bool includeRatsnest);
-	virtual void transformItem2(const QMatrix &);
+	virtual void transformItem2(const QTransform &);
 	virtual void removeLayerKin();
 	ViewLayer::ViewID viewID();
 	QString & viewIDName();
@@ -96,26 +112,27 @@ public:
 
 	void collectConnectors(ConnectorPairHash & connectorHash, SkipCheckFunction);
 
-	virtual void busConnectorItems(class Bus * bus, ConnectorItem *, QList<ConnectorItem *> & items);
+	virtual void busConnectorItems(Bus * bus, ConnectorItem *, QList<ConnectorItem *> & items);
 	virtual void setHidden(bool hidden);
 	virtual void setLayerHidden(bool hidden);
-	bool hidden();
-	bool layerHidden();
+	constexpr bool hidden() const noexcept { return m_hidden; }
+	constexpr bool layerHidden() const noexcept { return m_layerHidden; }
 	virtual void setInactive(bool inactivate);
-	bool inactive();
+	constexpr bool inactive() const noexcept { return m_inactive; }
 	ConnectorItem * findConnectorItemWithSharedID(const QString & connectorID, ViewLayer::ViewLayerPlacement);
 	ConnectorItem * findConnectorItemWithSharedID(const QString & connectorID);
 	void updateConnections(ConnectorItem *, bool includeRatsnest, QList<ConnectorItem *> & already);
 	virtual void updateConnections(bool includeRatsnest, QList<ConnectorItem *> & already);
-	virtual const QString & title() const;
+	virtual const QString & title();
+	const QString & constTitle() const;
 	bool getRatsnest();
-	QList<class Bus *> buses();
+	QList<Bus *> buses();
 	int itemType() const;					// wanted this to return ModelPart::ItemType but couldn't figure out how to get it to compile
 	virtual bool isSticky();
 	virtual bool isBaseSticky();
 	virtual void setSticky(bool);
 	void setLocalSticky(bool);
-    bool isLocalSticky();
+	bool isLocalSticky();
 	virtual void addSticky(ItemBase *, bool stickem);
 	virtual ItemBase * stickingTo();
 	virtual QList< QPointer<ItemBase> > stickyList();
@@ -140,7 +157,7 @@ public:
 	ViewLayer::ViewLayerID partLabelViewLayerID();
 	void clearPartLabel();
 	bool isPartLabelVisible();
-	void restorePartLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID);				// on loading from a file
+	void restorePartLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID, bool flipAware = false);				// on loading from a file
 	void movePartLabel(QPointF newPos, QPointF newOffset);												// coming down from the command object
 	void partLabelMoved(QPointF oldPos, QPointF oldOffset, QPointF newPos, QPointF newOffset);			// coming up from the label
 	void partLabelSetHidden(bool hide);
@@ -150,16 +167,17 @@ public:
 	QPointF partLabelScenePos();
 	QRectF partLabelSceneBoundingRect();
 	virtual bool isSwappable();
-    virtual void setSwappable(bool);
+	virtual void setSwappable(bool);
+	virtual bool allowSwapReconnectByDescription();
 	void mousePressEvent(QGraphicsSceneMouseEvent *event);
-	virtual void collectWireConnectees(QSet<class Wire *> & wires);
+	virtual void collectWireConnectees(QSet<Wire *> & wires);
 	virtual bool collectFemaleConnectees(QSet<ItemBase *> & items);
 	void prepareGeometryChange();
 	virtual void resetID();
 	void updateConnectionsAux(bool includeRatsnest, QList<ConnectorItem *> & already);
 	void hoverEnterEvent( QGraphicsSceneHoverEvent * event );
 	void hoverLeaveEvent( QGraphicsSceneHoverEvent * event );
-    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
+	void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
 	virtual void figureHover();
 	virtual QString retrieveSvg(ViewLayer::ViewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor);
 	virtual void slamZ(double newZ);
@@ -174,7 +192,7 @@ public:
 	virtual void setProp(const QString & prop, const QString & value);
 	QString prop(const QString & p);
 	bool isObsolete();
-    virtual QHash<QString, QString> prepareProps(ModelPart *, bool wantDebug, QStringList & keys);
+	virtual QHash<QString, QString> prepareProps(ModelPart *, bool wantDebug, QStringList & keys);
 	void resetValues(const QString & family, const QString & prop);
 	const QString & filename();
 	void setFilename(const QString &);
@@ -182,7 +200,7 @@ public:
 	ViewLayer::ViewLayerPlacement viewLayerPlacement() const;
 	void setViewLayerPlacement(ViewLayer::ViewLayerPlacement);
 	virtual void calcRotation(QTransform & rotation, QPointF center, ViewGeometry &);
-    void updateConnectors();
+	void updateConnectors();
 	const QString & moduleID();
 	bool moveLock();
 	virtual void setMoveLock(bool);
@@ -204,37 +222,39 @@ public:
 	const QList<ConnectorItem *> & cachedConnectorItemsConst() const;
 	bool inHover();
 	virtual QRectF boundingRectWithoutLegs() const;
-    QRectF boundingRect() const;
-    virtual QPainterPath hoverShape() const;
+	QRectF boundingRect() const;
+	virtual QPainterPath hoverShape() const;
 	virtual const QCursor * getCursor(Qt::KeyboardModifiers);
-	class PartLabel * partLabel();
+	PartLabel * partLabel();
 	virtual void doneLoading();
 	QString family();
 	QPixmap * getPixmap(QSize size);
-    class FSvgRenderer * fsvgRenderer() const;
-	void setSharedRendererEx(class FSvgRenderer *);
+	FSvgRenderer * fsvgRenderer() const;
+	void setSharedRendererEx(FSvgRenderer *);
 	bool reloadRenderer(const QString & svg, bool fastload);
 	bool resetRenderer(const QString & svg);
 	bool resetRenderer(const QString & svg, QString & newSvg);
-    void getPixmaps(QPixmap * &, QPixmap * &, QPixmap * &, bool swappingEnabled, QSize);
-    class FSvgRenderer * setUpImage(class ModelPart * modelPart, class LayerAttributes &);
+	void getPixmaps(QPixmap * &, QPixmap * &, QPixmap * &, bool swappingEnabled, QSize);
+	FSvgRenderer * setUpImage(ModelPart * modelPart, LayerAttributes &);
 	void showConnectors(const QStringList &);
 	void setItemIsSelectable(bool selectable);
 	virtual bool inRotation();
 	virtual void setInRotation(bool);
 	const QString & spice() const;
 	const QString & spiceModel() const;
-    void addSubpart(ItemBase *);
-    void setSuperpart(ItemBase *);
-    ItemBase * superpart();
-    ItemBase * findSubpart(const QString & connectorID, ViewLayer::ViewLayerPlacement);
-    const QList< QPointer<ItemBase> > & subparts();
-    void setSquashShape(bool);
-    const QPainterPath & selectionShape();
-    virtual void setTransform2(const QTransform &);
-    void initLayerAttributes(LayerAttributes & layerAttributes, ViewLayer::ViewID, ViewLayer::ViewLayerID, ViewLayer::ViewLayerPlacement, bool doConnectors, bool doCreateShape);
-    virtual QString getInspectorTitle();
-    virtual void setInspectorTitle(const QString & oldText, const QString & newText);
+	void addSubpart(ItemBase *);
+	void setSuperpart(ItemBase *);
+	ItemBase * superpart();
+	ItemBase * findSubpart(const QString & connectorID, ViewLayer::ViewLayerPlacement);
+	const QList< QPointer<ItemBase> > & subparts();
+	void setSquashShape(bool);
+	const QPainterPath & selectionShape();
+	virtual void setTransform2(const QTransform &);
+	void initLayerAttributes(LayerAttributes & layerAttributes, ViewLayer::ViewID, ViewLayer::ViewLayerID, ViewLayer::ViewLayerPlacement, bool doConnectors, bool doCreateShape);
+	virtual QString getInspectorTitle();
+	virtual void setInspectorTitle(const QString & oldText, const QString & newText);
+	void addSimulationGraphicsItem(QGraphicsObject *);
+	void removeSimulationGraphicsItem();
 
 public:
 	virtual void getConnectedColor(ConnectorItem *, QBrush &, QPen &, double & opacity, double & negativePenWidth, bool & negativeOffsetRect);
@@ -256,7 +276,7 @@ protected:
 	static QBrush UnconnectedBrush;
 	static QBrush ChosenBrush;
 	static QBrush EqualPotentialBrush;
-	static const double NormalConnectorOpacity;
+	static constexpr double NormalConnectorOpacity = 0.4;
 
 public:
 	static QColor connectedColor();
@@ -269,7 +289,7 @@ public:
 public:
 	virtual void hoverEnterConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
 	virtual void hoverLeaveConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
-	virtual void hoverMoveConnectorItem(QGraphicsSceneHoverEvent * event, class ConnectorItem * item);
+	virtual void hoverMoveConnectorItem(QGraphicsSceneHoverEvent * event, ConnectorItem * item);
 	void hoverEnterConnectorItem();
 	void hoverLeaveConnectorItem();
 	virtual void connectorHover(ConnectorItem *, ItemBase *, bool hovering);
@@ -291,18 +311,20 @@ public:
 	virtual ItemBase * layerKinChief();
 	virtual const QList<ItemBase *> & layerKin();
 	virtual void findConnectorsUnder() = 0;
-	virtual ConnectorItem* newConnectorItem(class Connector *connector);
+	virtual ConnectorItem* newConnectorItem(Connector *connector);
 	virtual ConnectorItem* newConnectorItem(ItemBase * layerkin, Connector *connector);
 
 	virtual void setInstanceTitle(const QString &title, bool initial);
 	void updatePartLabelInstanceTitle();
+	std::pair<QString, bool> migratePartLabel();
 
-public slots:
+public Q_SLOTS:
 	void showPartLabel(bool show, ViewLayer *);
 	void hidePartLabel();
 	void partLabelChanged(const QString &newText);
 	virtual void swapEntry(const QString & text);
-    void showInFolder();
+	virtual void swapEntry(int index);
+	void showInFolder();
 
 public:
 	static bool zLessThan(ItemBase * & p1, ItemBase * & p2);
@@ -312,9 +334,9 @@ public:
 protected:
 	void mouseMoveEvent(QGraphicsSceneMouseEvent *event);
 	void mouseReleaseEvent(QGraphicsSceneMouseEvent *event );
-	virtual void paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget); 
-	virtual void paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget, const QPainterPath & shape); 
-    virtual void paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
+	virtual void paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
+	virtual void paintHover(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget, const QPainterPath & shape);
+	virtual void paintSelected(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
 	virtual void paintBody(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
 
 	QVariant itemChange(QGraphicsItem::GraphicsItemChange change, const QVariant & value);
@@ -325,66 +347,67 @@ protected:
 	virtual void setDefaultTooltip();
 	void setInstanceTitleAux(const QString & title, bool initial);
 	void saveLocAndTransform(QXmlStreamWriter & streamWriter);
-    QPixmap * getPixmap(ViewLayer::ViewID, bool swappingEnabled, QSize size);
-    virtual ViewLayer::ViewID useViewIDForPixmap(ViewLayer::ViewID, bool swappingEnabled);
-    virtual bool makeLocalModifications(QByteArray & svg, const QString & filename);
-    void updateHidden();
-    void createShape(LayerAttributes & layerAttributes);
+	QPixmap * getPixmap(ViewLayer::ViewID, bool swappingEnabled, QSize size);
+	virtual ViewLayer::ViewID useViewIDForPixmap(ViewLayer::ViewID, bool swappingEnabled);
+	virtual bool makeLocalModifications(QByteArray & svg, const QString & filename);
+	void updateHidden();
+	virtual void createShape(LayerAttributes & layerAttributes);
 
 protected:
 	static bool getFlipDoc(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerPlacement, QDomDocument &, Qt::Orientations);
 	static bool fixCopper1(ModelPart * modelPart, const QString & filename, ViewLayer::ViewLayerID viewLayerID, ViewLayer::ViewLayerPlacement, QDomDocument &);
 
 protected:
- 	QSizeF m_size;
-	qint64 m_id;
+	QSizeF m_size;
+	qint64 m_id = 0;
 	ViewGeometry m_viewGeometry;
 	QPointer<ModelPart> m_modelPart;
 	ViewLayer::ViewID m_viewID;
 	ViewLayer::ViewLayerID m_viewLayerID;
-	int m_connectorHoverCount;
-	int m_connectorHoverCount2;
-	int m_hoverCount;
-	bool m_hidden;
-	bool m_layerHidden;
-	bool m_inactive;
-	bool m_sticky;
+	int m_connectorHoverCount = 0;
+	int m_connectorHoverCount2 = 0;
+	int m_hoverCount = 0;
+	bool m_hidden = false;
+	bool m_layerHidden = false;
+	bool m_inactive = false;
+	bool m_sticky = false;
 	QHash< long, QPointer<ItemBase> > m_stickyList;
-	QMenu *m_itemMenu;
-	bool m_canFlipHorizontal;
-	bool m_canFlipVertical;
-	bool m_zUninitialized;
-	QPointer<class PartLabel> m_partLabel;
-	bool m_spaceBarWasPressed;
-	bool m_hoverEnterSpaceBarWasPressed;
-	bool m_everVisible;
-	ConnectorItem * m_rightClickedConnector;
+	QMenu *m_itemMenu = nullptr;
+	bool m_canFlipHorizontal = false;
+	bool m_canFlipVertical = false;
+	bool m_zUninitialized = true;
+	QPointer<PartLabel> m_partLabel;
+	bool m_spaceBarWasPressed = false;
+	bool m_hoverEnterSpaceBarWasPressed = false;
+	bool m_everVisible = true;
+	ConnectorItem * m_rightClickedConnector = nullptr;
 	QMap<QString, QString> m_propsMap;
 	QString m_filename;
 	ViewLayer::ViewLayerPlacement m_viewLayerPlacement;
-	bool m_moveLock;
-	bool m_hasRubberBandLeg;
+	bool m_moveLock = false;
+	bool m_hasRubberBandLeg = false;
 	QList<ConnectorItem *> m_cachedConnectorItems;
-	QGraphicsSvgItem * m_moveLockItem;
-	QGraphicsSvgItem * m_stickyItem;
-    FSvgRenderer * m_fsvgRenderer;
-	bool m_acceptsMousePressLegEvent;
-    bool m_swappable;
- 	bool m_inRotation;
-    QPointer<ItemBase> m_superpart;
-    QList< QPointer<ItemBase> > m_subparts;
-    bool m_squashShape;
-    QPainterPath m_selectionShape;
-      
- protected:
+	QGraphicsSvgItem * m_moveLockItem = nullptr;
+	QGraphicsSvgItem * m_stickyItem = nullptr;
+	FSvgRenderer * m_fsvgRenderer = nullptr;
+	bool m_acceptsMousePressLegEvent = true;
+	bool m_swappable = true;
+	bool m_inRotation = false;
+	QPointer<ItemBase> m_superpart;
+	QList< QPointer<ItemBase> > m_subparts;
+	bool m_squashShape = false;
+	QPainterPath m_selectionShape;
+	QGraphicsObject * m_simItem = nullptr;
+
+protected:
 	static long nextID;
-	static QPointer<class ReferenceModel> TheReferenceModel;
+	static QPointer<ReferenceModel> TheReferenceModel;
 
 public:
 	static const QString ITEMBASE_FONT_PREFIX;
 	static const QString ITEMBASE_FONT_SUFFIX;
 	static QHash<QString, QString> TranslatedPropertyNames;
-    static QString PartInstanceDefaultTitle;
+	static QString PartInstanceDefaultTitle;
 	static const QList<ItemBase *> EmptyList;
 	const static QColor HoverColor;
 	const static double HoverOpacity;
@@ -396,9 +419,8 @@ public:
 	static void cleanup();
 	static ItemBase * extractTopLevelItemBase(QGraphicsItem * thing);
 	static QString translatePropertyName(const QString & key);
-	static void setReferenceModel(class ReferenceModel *);
-    static void renderOne(QDomDocument *, QImage *, const QRectF & renderRect);
-
+	static void setReferenceModel(ReferenceModel *);
+	static void renderOne(QDomDocument *, QImage *, const QRectF & renderRect);
 
 
 };
