@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,12 +16,6 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6980 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-04-22 01:45:43 +0200 (Mo, 22. Apr 2013) $
-
 ********************************************************************/
 
 #include "note.h"
@@ -31,7 +25,6 @@ $Date: 2013-04-22 01:45:43 +0200 (Mo, 22. Apr 2013) $
 #include "../utils/resizehandle.h"
 #include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
-#include "../fsvgrenderer.h"
 
 #include <QTextFrame>
 #include <QTextLayout>
@@ -74,18 +67,18 @@ const int Note::emptyMinWidth = 40;
 const int Note::emptyMinHeight = 25;
 const int Note::initialMinWidth = 140;
 const int Note::initialMinHeight = 45;
-const int borderWidth = 7;
-const int TriangleOffset = 7;
+constexpr int borderWidth = 7;
+constexpr int TriangleOffset = 7;
 
-const double InactiveOpacity = 0.5;
+constexpr double InactiveOpacity = 0.5;
 
 QString Note::initialTextString;
 
-QRegExp UrlTag("<a.*href=[\"']([^\"]+[.\\s]*)[\"'].*>");    
+QRegularExpression UrlTag("<a.*href=[\"']([^\"]+[.\\s]*)[\"'].*>");
 
 ///////////////////////////////////////
 
-void findA(QDomElement element, QList<QDomElement> & aElements) 
+void findA(QDomElement element, QList<QDomElement> & aElements)
 {
 	if (element.tagName().compare("a", Qt::CaseInsensitive) == 0) {
 		aElements.append(element);
@@ -101,28 +94,18 @@ void findA(QDomElement element, QList<QDomElement> & aElements)
 }
 
 
-QString addText(const QString & text, bool inUrl) 
+QString addText(const QString & text, bool inUrl)
 {
 	if (text.isEmpty()) return "";
 
 	return QString("<tspan fill='%1' >%2</tspan>\n")
-				.arg(inUrl ? "#0000ff" : "#000000")
-				.arg(TextUtils::convertExtendedChars(TextUtils::escapeAnd(text)))
-				;
+	       .arg(inUrl ? "#0000ff" : "#000000")
+	       .arg(TextUtils::convertExtendedChars(TextUtils::escapeAnd(text)))
+	       ;
 }
 
 ///////////////////////////////////////
 
-class NoteGraphicsTextItem : public QGraphicsTextItem
-{
-public:
-	NoteGraphicsTextItem(QGraphicsItem * parent = NULL);
-
-protected:
-	void focusInEvent(QFocusEvent *);
-	void focusOutEvent(QFocusEvent *);
-};
-
 NoteGraphicsTextItem::NoteGraphicsTextItem(QGraphicsItem * parent) : QGraphicsTextItem(parent)
 {
 	const QTextFrameFormat format = document()->rootFrame()->frameFormat();
@@ -131,9 +114,12 @@ NoteGraphicsTextItem::NoteGraphicsTextItem(QGraphicsItem * parent) : QGraphicsTe
 	document()->rootFrame()->setFrameFormat(altFormat);
 }
 
+void NoteGraphicsTextItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *) {
+}
+
 void NoteGraphicsTextItem::focusInEvent(QFocusEvent * event) {
 	InfoGraphicsView * igv = InfoGraphicsView::getInfoGraphicsView(this);
-	if (igv != NULL) {
+	if (igv != nullptr) {
 		igv->setNoteFocus(this, true);
 	}
 	QApplication::instance()->installEventFilter((Note *) this->parentItem());
@@ -143,7 +129,7 @@ void NoteGraphicsTextItem::focusInEvent(QFocusEvent * event) {
 
 void NoteGraphicsTextItem::focusOutEvent(QFocusEvent * event) {
 	InfoGraphicsView * igv = InfoGraphicsView::getInfoGraphicsView(this);
-	if (igv != NULL) {
+	if (igv != nullptr) {
 		igv->setNoteFocus(this, false);
 	}
 	QApplication::instance()->removeEventFilter((Note *) this->parentItem());
@@ -153,15 +139,15 @@ void NoteGraphicsTextItem::focusOutEvent(QFocusEvent * event) {
 
 //////////////////////////////////////////
 
-LinkDialog::LinkDialog(QWidget *parent) : QDialog(parent) 
+LinkDialog::LinkDialog(QWidget *parent) : QDialog(parent)
 {
 	this->setWindowTitle(QObject::tr("Edit link"));
 
-	QVBoxLayout * vLayout = new QVBoxLayout(this);
+	auto * vLayout = new QVBoxLayout(this);
 
-	QGroupBox * formGroupBox = new QGroupBox(this);
+	auto * formGroupBox = new QGroupBox(this);
 
-	QFormLayout * formLayout = new QFormLayout();
+	auto * formLayout = new QFormLayout();
 
 	m_urlEdit = new QLineEdit(this);
 	m_urlEdit->setFixedHeight(25);
@@ -177,12 +163,12 @@ LinkDialog::LinkDialog(QWidget *parent) : QDialog(parent)
 
 	vLayout->addWidget(formGroupBox);
 
-    QDialogButtonBox * buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	auto * buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
 	buttonBox->button(QDialogButtonBox::Cancel)->setText(tr("Cancel"));
 	buttonBox->button(QDialogButtonBox::Ok)->setText(tr("OK"));
 
-    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
-    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
+	connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
+	connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 
 	vLayout->addWidget(buttonBox);
 
@@ -218,10 +204,10 @@ Note::Note( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry
 		initialTextString = tr("[write your note here]");
 	}
 
-	m_inResize = NULL;
+	m_inResize = nullptr;
 	this->setCursor(Qt::ArrowCursor);
 
-    setFlag(QGraphicsItem::ItemIsSelectable, true);
+	setFlag(QGraphicsItem::ItemIsSelectable, true);
 
 	if (viewGeometry.rect().width() == 0 || viewGeometry.rect().height() == 0) {
 		m_rect.setRect(0, 0, Note::initialMinWidth, Note::initialMinHeight);
@@ -231,9 +217,9 @@ Note::Note( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry
 	}
 	m_pen.setWidth(borderWidth);
 	m_pen.setCosmetic(false);
-    m_pen.setBrush(QColor(0xfa, 0xbc, 0x4f));
+	m_pen.setBrush(QColor(0xfa, 0xbc, 0x4f));
 
-    m_brush.setColor(QColor(0xff, 0xe9, 0xc8));
+	m_brush.setColor(QColor(0xff, 0xe9, 0xc8));
 	m_brush.setStyle(Qt::SolidPattern);
 
 	setPos(m_viewGeometry.loc());
@@ -248,6 +234,7 @@ Note::Note( ModelPart * modelPart, ViewLayer::ViewID viewID,  const ViewGeometry
 	m_graphicsTextItem = new NoteGraphicsTextItem();
 	QFont font("Droid Sans", 9, QFont::Normal);
 	m_graphicsTextItem->setFont(font);
+	m_graphicsTextItem->setDefaultTextColor(QColor("grey"));
 	m_graphicsTextItem->document()->setDefaultFont(font);
 	m_graphicsTextItem->setParentItem(this);
 	m_graphicsTextItem->setVisible(true);
@@ -304,30 +291,30 @@ void Note::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWid
 
 	painter->setPen(Qt::NoPen);
 	painter->setBrush(m_brush);
-    QPolygonF poly;
-    poly.append(m_rect.topLeft());
-    poly.append(m_rect.topRight() + QPointF(-TriangleOffset, 0));
-    poly.append(m_rect.topRight() + QPointF(0, TriangleOffset));
-    poly.append(m_rect.bottomRight());
-    poly.append(m_rect.bottomLeft());
-    painter->drawPolygon(poly);
+	QPolygonF poly;
+	poly.append(m_rect.topLeft());
+	poly.append(m_rect.topRight() + QPointF(-TriangleOffset, 0));
+	poly.append(m_rect.topRight() + QPointF(0, TriangleOffset));
+	poly.append(m_rect.bottomRight());
+	poly.append(m_rect.bottomLeft());
+	painter->drawPolygon(poly);
 
-    painter->setBrush(m_pen.brush());
-    poly.clear();
-    poly.append(m_rect.topRight() + QPointF(-TriangleOffset, 0));
-    poly.append(m_rect.topRight() + QPointF(0, TriangleOffset));
-    poly.append(m_rect.topRight() + QPointF(-TriangleOffset, TriangleOffset));
-    painter->drawPolygon(poly);
+	painter->setBrush(m_pen.brush());
+	poly.clear();
+	poly.append(m_rect.topRight() + QPointF(-TriangleOffset, 0));
+	poly.append(m_rect.topRight() + QPointF(0, TriangleOffset));
+	poly.append(m_rect.topRight() + QPointF(-TriangleOffset, TriangleOffset));
+	painter->drawPolygon(poly);
 
-    painter->setPen(m_pen);
-    poly.clear();
-    poly.append(QPointF(0, m_rect.bottom() - borderWidth / 2.0));
-    poly.append(QPointF(m_rect.right(), m_rect.bottom() - borderWidth / 2.0));
-    painter->drawPolygon(poly);
+	painter->setPen(m_pen);
+	poly.clear();
+	poly.append(QPointF(0, m_rect.bottom() - borderWidth / 2.0));
+	poly.append(QPointF(m_rect.right(), m_rect.bottom() - borderWidth / 2.0));
+	painter->drawPolygon(poly);
 
 	if (option->state & QStyle::State_Selected) {
 		GraphicsUtils::qt_graphicsItem_highlightSelected(painter, option, boundingRect(), QPainterPath());
-    }
+	}
 
 	if (m_inactive) {
 		painter->restore();
@@ -341,14 +328,14 @@ QRectF Note::boundingRect() const
 
 QPainterPath Note::shape() const
 {
-    QPainterPath path;
-    path.addRect(boundingRect());
-    return path;
+	QPainterPath path;
+	path.addRect(boundingRect());
+	return path;
 }
 
 void Note::positionGrip() {
 	QSizeF gripSize = m_resizeGrip->boundingRect().size();
-	QSizeF sz = this->boundingRect().size(); 
+	QSizeF sz = Note::boundingRect().size();
 	QPointF p(sz.width() - gripSize.width(), sz.height() - gripSize.height());
 	m_resizeGrip->setPos(p);
 	m_graphicsTextItem->setPos(TriangleOffset / 2, TriangleOffset / 2);
@@ -357,14 +344,14 @@ void Note::positionGrip() {
 
 void Note::mousePressEvent(QGraphicsSceneMouseEvent * event) {
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infographics != NULL && infographics->spaceBarIsPressed()) {
+	if ((infographics != nullptr) && infographics->spaceBarIsPressed()) {
 		m_spaceBarWasPressed = true;
 		event->ignore();
 		return;
 	}
 
 	m_spaceBarWasPressed = false;
-	m_inResize = NULL;
+	m_inResize = nullptr;
 	ItemBase::mousePressEvent(event);
 }
 
@@ -414,8 +401,8 @@ void Note::forceFormat(int position, int charsAdded) {
 	//textCursor.setCharFormat(f);
 	textCursor.mergeCharFormat(f);
 	//DebugDialog::debug(QString("setting font tc:%1,%2 params:%3,%4")
-		//.arg(textCursor.anchor()).arg(textCursor.position())
-		//.arg(position).arg(position + charsAdded));
+	//.arg(textCursor.anchor()).arg(textCursor.position())
+	//.arg(position).arg(position + charsAdded));
 
 	/*
 	textCursor = m_graphicsTextItem->textCursor();
@@ -437,15 +424,15 @@ void Note::forceFormat(int position, int charsAdded) {
 
 void Note::contentsChangedSlot() {
 
-    //DebugDialog::debug(QString("contents changed ") + m_graphicsTextItem->document()->toPlainText());
+	//DebugDialog::debug(QString("contents changed ") + m_graphicsTextItem->document()->toPlainText());
 	if (m_charsAdded > 0) {
 		forceFormat(m_charsPosition, m_charsAdded);
 	}
 
 	InfoGraphicsView *infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infoGraphicsView != NULL) {
+	if (infoGraphicsView != nullptr) {
 		QString oldText;
-		if (m_modelPart) {
+		if (m_modelPart != nullptr) {
 			oldText = m_modelPart->instanceText();
 		}
 
@@ -455,7 +442,7 @@ void Note::contentsChangedSlot() {
 
 		infoGraphicsView->noteChanged(this, oldText, m_graphicsTextItem->document()->toHtml(), oldSize, newSize);
 	}
-	if (m_modelPart) {
+	if (m_modelPart != nullptr) {
 		m_modelPart->setInstanceText(m_graphicsTextItem->document()->toHtml());
 	}
 }
@@ -474,16 +461,16 @@ void Note::checkSize(QSizeF & newSize) {
 
 void Note::disconnectSlots() {
 	disconnect(m_graphicsTextItem->document(), SIGNAL(contentsChanged()),
-			this, SLOT(contentsChangedSlot()));
+	           this, SLOT(contentsChangedSlot()));
 	disconnect(m_graphicsTextItem->document(), SIGNAL(contentsChange(int, int, int)),
-			this, SLOT(contentsChangeSlot(int, int, int)));
+	           this, SLOT(contentsChangeSlot(int, int, int)));
 }
 
 void Note::connectSlots() {
 	connect(m_graphicsTextItem->document(), SIGNAL(contentsChanged()),
-		this, SLOT(contentsChangedSlot()), Qt::DirectConnection);
+	        this, SLOT(contentsChangedSlot()), Qt::DirectConnection);
 	connect(m_graphicsTextItem->document(), SIGNAL(contentsChange(int, int, int)),
-		this, SLOT(contentsChangeSlot(int, int, int)), Qt::DirectConnection);
+	        this, SLOT(contentsChangeSlot(int, int, int)), Qt::DirectConnection);
 }
 
 void Note::setText(const QString & text, bool check) {
@@ -536,7 +523,7 @@ bool Note::eventFilter(QObject * object, QEvent * event)
 	}
 
 	if (event->type() == QEvent::KeyPress) {
-		QKeyEvent * kevent = static_cast<QKeyEvent *>(event);
+		auto * kevent = static_cast<QKeyEvent *>(event);
 		if (kevent->matches(QKeySequence::Bold)) {
 			QTextCursor textCursor = m_graphicsTextItem->textCursor();
 			QTextCharFormat cf = textCursor.charFormat();
@@ -556,7 +543,7 @@ bool Note::eventFilter(QObject * object, QEvent * event)
 			event->accept();
 			return true;
 		}
-		if ((kevent->key() == Qt::Key_L) && (kevent->modifiers() & Qt::ControlModifier)) {
+		if ((kevent->key() == Qt::Key_L) && ((kevent->modifiers() & Qt::ControlModifier) != 0u)) {
 			QTimer::singleShot(75, this, SLOT(linkDialog()));
 			event->accept();
 			return true;
@@ -569,9 +556,10 @@ bool Note::eventFilter(QObject * object, QEvent * event)
 void Note::linkDialog() {
 	QTextCursor textCursor = m_graphicsTextItem->textCursor();
 	bool gotUrl = false;
+	QRegularExpressionMatch match;
 	if (textCursor.anchor() == textCursor.selectionStart()) {
 		// the selection returns empty since we're between characters
-		// so select one character forward or one character backward 
+		// so select one character forward or one character backward
 		// to see whether we're in a url
 		int wasAnchor = textCursor.anchor();
 		bool atEnd = textCursor.atEnd();
@@ -579,14 +567,16 @@ void Note::linkDialog() {
 		if (!atStart) {
 			textCursor.setPosition(wasAnchor - 1, QTextCursor::KeepAnchor);
 			QString html = textCursor.selection().toHtml();
-			if (UrlTag.indexIn(html) >= 0) {
+			if (html.contains(UrlTag, &match)) {
 				gotUrl = true;
+			} else {
+				match = QRegularExpressionMatch();
 			}
 		}
 		if (!gotUrl && !atEnd) {
 			textCursor.setPosition(wasAnchor + 1, QTextCursor::KeepAnchor);
 			QString html = textCursor.selection().toHtml();
-			if (UrlTag.indexIn(html) >= 0) {
+			if (html.contains(UrlTag, &match)) {
 				gotUrl = true;
 			}
 		}
@@ -595,7 +585,7 @@ void Note::linkDialog() {
 	else {
 		QString html = textCursor.selection().toHtml();
 		DebugDialog::debug(html);
-		if (UrlTag.indexIn(html) >= 0) {
+		if (html.contains(UrlTag, &match)) {
 			gotUrl = true;
 		}
 	}
@@ -604,7 +594,7 @@ void Note::linkDialog() {
 	QString originalText;
 	QString originalUrl;
 	if (gotUrl) {
-		originalUrl = UrlTag.cap(1);
+		originalUrl = match.captured(1);
 		ld.setUrl(originalUrl);
 		QString html = m_graphicsTextItem->toHtml();
 
@@ -630,7 +620,7 @@ void Note::linkDialog() {
 		DebugDialog::debug(html);
 		QList<QDomElement> aElements;
 		findA(root, aElements);
-		foreach (QDomElement a, aElements) {
+		Q_FOREACH (QDomElement a, aElements) {
 			// TODO: if multiple hrefs point to the same url this will only find the first one
 			QString href = a.attribute("href");
 			if (href.isEmpty()) {
@@ -691,9 +681,9 @@ void Note::handleMousePressSlot(QGraphicsSceneMouseEvent * event, ResizeHandle *
 }
 
 void Note::handleMouseMoveSlot(QGraphicsSceneMouseEvent * event, ResizeHandle * resizeHandle) {
-    Q_UNUSED(resizeHandle);
+	Q_UNUSED(resizeHandle);
 
-    if (!m_inResize) return;
+	if (m_inResize == nullptr) return;
 
 	double minWidth = emptyMinWidth;
 	double minHeight = emptyMinHeight;
@@ -715,29 +705,29 @@ void Note::handleMouseMoveSlot(QGraphicsSceneMouseEvent * event, ResizeHandle *
 	}
 	if (newY - oldY1 < minHeight) {
 		newY = oldY1 + minHeight;
-	}	
+	}
 	newR.setRect(0, 0, newX - oldX1, newY - oldY1);
 
 	prepareGeometryChange();
 	m_rect = newR;
-	positionGrip();	
+	positionGrip();
 }
 
 void Note::handleMouseReleaseSlot(QGraphicsSceneMouseEvent * event, ResizeHandle * resizeHandle) {
-    Q_UNUSED(resizeHandle);
-    Q_UNUSED(event);
+	Q_UNUSED(resizeHandle);
+	Q_UNUSED(event);
 
-	if (!m_inResize) return;
+	if (m_inResize == nullptr) return;
 
-	m_inResize = NULL;
+	m_inResize = nullptr;
 	InfoGraphicsView *infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infoGraphicsView != NULL) {
-		infoGraphicsView->noteSizeChanged(this, m_viewGeometry.rect(), m_rect);
+	if (infoGraphicsView != nullptr) {
+		infoGraphicsView->noteSizeChanged(this, m_viewGeometry.rect().size(), m_rect.size());
 	}
 }
 
 bool Note::hasPartLabel() {
-	
+
 	return false;
 }
 
@@ -761,20 +751,20 @@ bool Note::rotation45Allowed() {
 QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor)
 {
 	Q_UNUSED(svgHash);
-    factor = 1;
+	factor = 1;
 
 	switch (viewLayerID) {
-		case ViewLayer::BreadboardNote:
-			if (viewID() != ViewLayer::BreadboardView) return "";
-			break;
-		case ViewLayer::PcbNote:
-			if (viewID() != ViewLayer::PCBView) return "";
-			break;
-		case ViewLayer::SchematicNote:
-			if (viewID() != ViewLayer::SchematicView) return "";
-			break;
-		default:
-			return "";
+	case ViewLayer::BreadboardNote:
+		if (viewID() != ViewLayer::BreadboardView) return "";
+		break;
+	case ViewLayer::PcbNote:
+		if (viewID() != ViewLayer::PCBView) return "";
+		break;
+	case ViewLayer::SchematicNote:
+		if (viewID() != ViewLayer::SchematicView) return "";
+		break;
+	default:
+		return "";
 	}
 
 	QString svg = "<g>";
@@ -783,14 +773,14 @@ QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QSt
 	double penWidth = m_pen.widthF() * dpi / GraphicsUtils::SVGDPI;
 	QString brushColor = blackOnly ? "none" : m_brush.color().name();
 	svg += QString("<rect x='%1' y='%2' width='%3' height='%4' fill='%5' stroke='%6' stroke-width='%7' />")
-		.arg(penWidth / 2)
-		.arg(penWidth / 2)
-		.arg((m_rect.width() * dpi / GraphicsUtils::SVGDPI) - penWidth)
-		.arg((m_rect.height() * dpi / GraphicsUtils::SVGDPI) - penWidth)
-		.arg(brushColor)
-		.arg(penColor)
-		.arg(penWidth)
-		;
+	       .arg(penWidth / 2)
+	       .arg(penWidth / 2)
+	       .arg((m_rect.width() * dpi / GraphicsUtils::SVGDPI) - penWidth)
+	       .arg((m_rect.height() * dpi / GraphicsUtils::SVGDPI) - penWidth)
+	       .arg(brushColor)
+	       .arg(penColor)
+	       .arg(penWidth)
+	       ;
 
 	QTextCursor textCursor = m_graphicsTextItem->textCursor();
 	QSizeF gripSize = m_resizeGrip->boundingRect().size();
@@ -809,11 +799,11 @@ QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QSt
 			QString soFar;
 
 			svg += QString("<text  x='%1' y='%2' font-family='%3' stroke='none' fill='#000000' text-anchor='left' font-size='%4' >\n")
-				.arg((left + r.left()) * dpi / GraphicsUtils::SVGDPI)
-				.arg((top + r.top() + line.ascent()) * dpi / GraphicsUtils::SVGDPI)
-				.arg("Droid Sans")
-				.arg(line.ascent() * dpi / GraphicsUtils::SVGDPI) 
-				;
+			       .arg((left + r.left()) * dpi / GraphicsUtils::SVGDPI)
+			       .arg((top + r.top() + line.ascent()) * dpi / GraphicsUtils::SVGDPI)
+			       .arg("Droid Sans")
+			       .arg(line.ascent() * dpi / GraphicsUtils::SVGDPI)
+			       ;
 
 
 			bool inUrl = false;
@@ -821,7 +811,7 @@ QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QSt
 				textCursor.setPosition(i + start + block.position(), QTextCursor::MoveAnchor);
 				textCursor.setPosition(i + start + block.position() + 1, QTextCursor::KeepAnchor);
 				QString html = textCursor.selection().toHtml();
-				if (UrlTag.indexIn(html) >= 0) {
+				if (html.contains(UrlTag)) {
 					if (inUrl) {
 						soFar += block.text().mid(start + i, 1);
 						continue;
@@ -845,23 +835,23 @@ QString Note::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QSt
 			svg += addText(soFar, inUrl && !blackOnly);
 			svg += "</text>";
 		}
-	}	
+	}
 
 
 	svg += "</g>";
 
 
-	return svg;	
+	return svg;
 }
 
-ViewLayer::ViewID Note::useViewIDForPixmap(ViewLayer::ViewID, bool) 
+ViewLayer::ViewID Note::useViewIDForPixmap(ViewLayer::ViewID, bool)
 {
-    return ViewLayer::IconView;
+	return ViewLayer::IconView;
 }
 
 void Note::addedToScene(bool temporary)
 {
 	positionGrip();
 
-    return ItemBase::addedToScene(temporary);
+	return ItemBase::addedToScene(temporary);
 }
