@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,12 +16,6 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6998 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
-
 ********************************************************************/
 
 #include "paletteitembase.h"
@@ -35,7 +29,6 @@ $Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 #include "../connectors/svgidlayer.h"
 #include "wire.h"
 #include "partlabel.h"
-#include "../utils/textutils.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/cursormaster.h"
 
@@ -47,6 +40,7 @@ $Date: 2013-04-28 13:51:10 +0200 (So, 28. Apr 2013) $
 #include <QLineEdit>
 #include <QApplication>
 #include <QEvent>
+#include <QRegularExpressionValidator>
 #include <qmath.h>
 
 static QPointF RotationCenter;
@@ -60,19 +54,26 @@ PaletteItemBase::PaletteItemBase(ModelPart * modelPart, ViewLayer::ViewID viewID
 	m_syncSelected = false;
 	m_offset.setX(0);
 	m_offset.setY(0);
- 	m_blockItemSelectedChange = false;
+	m_blockItemSelectedChange = false;
 	this->setPos(viewGeometry.loc());
-    setFlag(QGraphicsItem::ItemIsSelectable, true);
+	setFlag(QGraphicsItem::ItemIsSelectable, true);
 	setFlag(QGraphicsItem::ItemSendsGeometryChanges, true);
 	setAcceptHoverEvents(true);
 
 	if (hasPartNumberProperty()) {
-		QString savedValue = modelPart->localProp(ModelPartShared::PartNumberPropertyName).toString();
-		if (savedValue.isEmpty()) {
-			savedValue = modelPart->properties().value(ModelPartShared::PartNumberPropertyName, "");
-			if (!savedValue.isEmpty()) {
-				modelPart->setLocalProp(ModelPartShared::PartNumberPropertyName, savedValue);
-			}
+		for (auto&& propertyName : {ModelPartShared::MNPropertyName, ModelPartShared::MPNPropertyName, ModelPartShared::PartNumberPropertyName}) {
+			initLocalProperty(propertyName, modelPart);
+		}
+	}
+}
+
+void PaletteItemBase::initLocalProperty(const QString & propertyName, ModelPart * modelPart)
+{
+	QString savedValue = modelPart->localProp(propertyName).toString();
+	if (savedValue.isEmpty()) {
+		savedValue = modelPart->properties().value(propertyName, "");
+		if (!savedValue.isEmpty()) {
+			modelPart->setLocalProp(propertyName, savedValue);
 		}
 	}
 }
@@ -95,12 +96,12 @@ QRectF PaletteItemBase::boundingRect() const
 QPainterPath PaletteItemBase::hoverShape() const
 {
 	// TODO: figure out real shape of svg
-    QPainterPath path;
-    path.addRect(0, 0, m_size.width(), m_size.height());
+	QPainterPath path;
+	path.addRect(0, 0, m_size.width(), m_size.height());
 
 	if (!hasRubberBandLeg()) return path;
 
-	foreach (ConnectorItem * connectorItem, cachedConnectorItemsConst()) {
+	Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItemsConst()) {
 		if (connectorItem->hasRubberBandLeg()) {
 			path.addPath(connectorItem->mapToParent(connectorItem->hoverShape()));
 		}
@@ -113,14 +114,14 @@ QPainterPath PaletteItemBase::hoverShape() const
 QPainterPath PaletteItemBase::shape() const
 {
 	// TODO: figure out real shape of svg
-    QPainterPath path;
-    if (m_squashShape) return path;
+	QPainterPath path;
+	if (m_squashShape) return path;
 
-    path.addRect(0, 0, m_size.width(), m_size.height());
+	path.addRect(0, 0, m_size.width(), m_size.height());
 
 	if (!hasRubberBandLeg()) return path;
 
-	foreach (ConnectorItem * connectorItem, cachedConnectorItemsConst()) {
+	Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItemsConst()) {
 		if (connectorItem->hasRubberBandLeg()) {
 			path.addPath(connectorItem->mapToParent(connectorItem->shape()));
 		}
@@ -142,7 +143,7 @@ bool PaletteItemBase::itemMoved() {
 
 void PaletteItemBase::moveItem(ViewGeometry & viewGeometry) {
 	this->setPos(viewGeometry.loc());
-    QList<ConnectorItem *> already;
+	QList<ConnectorItem *> already;
 	updateConnections(false, already);
 }
 
@@ -211,21 +212,21 @@ void PaletteItemBase::paintSelected(QPainter *painter, const QStyleOptionGraphic
 
 void PaletteItemBase::mousePressConnectorEvent(ConnectorItem * connectorItem, QGraphicsSceneMouseEvent * event) {
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infoGraphicsView != NULL) {
+	if (infoGraphicsView != nullptr) {
 		infoGraphicsView->mousePressConnectorEvent(connectorItem, event);
 	}
 }
 
 bool PaletteItemBase::acceptsMousePressConnectorEvent(ConnectorItem *, QGraphicsSceneMouseEvent * event) {
 	Q_UNUSED(event);
-	
+
 	//if (m_viewID != ViewLayer::PCBView) {
-		return true;
+	return true;
 	//}
 }
 
 
-bool PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
+bool PaletteItemBase::mousePressEventK(PaletteItemBase * originalItem, QGraphicsSceneMouseEvent *event)
 {
 	Q_UNUSED(originalItem);
 
@@ -236,7 +237,7 @@ bool PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 		this->saveGeometry();
 		setInRotation(true);
 		RotationCenter = mapToScene(this->boundingRectWithoutLegs().center());
-		RotationAxis = event->scenePos(); 
+		RotationAxis = event->scenePos();
 		OriginalTransform = this->transform();
 		/*
 		DebugDialog::debug(QString("%11:in rotation:%1,%2 a:%3,%4 t:%5,%6,%7,%8 %9,%10")
@@ -249,25 +250,25 @@ bool PaletteItemBase::mousePressEvent(PaletteItemBase * originalItem, QGraphicsS
 			*/
 		this->debugInfo("in rotation");
 		InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-		if (infoGraphicsView) infoGraphicsView->setAnyInRotation();
+		if (infoGraphicsView != nullptr) infoGraphicsView->setAnyInRotation();
 		return false;
 	}
 
 	ItemBase::mousePressEvent(event);
 	if (canFindConnectorsUnder()) {
-		foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
-			connectorItem->setOverConnectorItem(NULL);
+		Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItems()) {
+			connectorItem->setOverConnectorItem(nullptr);
 		}
 	}
 
-    return false;
+	return false;
 }
 
-void PaletteItemBase::mouseMoveEvent(PaletteItemBase *, QGraphicsSceneMouseEvent *)
+void PaletteItemBase::mouseMoveEventK(PaletteItemBase *, QGraphicsSceneMouseEvent *)
 {
 }
 
-void PaletteItemBase::mouseReleaseEvent(PaletteItemBase *, QGraphicsSceneMouseEvent *)
+void PaletteItemBase::mouseReleaseEventK(PaletteItemBase *, QGraphicsSceneMouseEvent *)
 {
 }
 
@@ -287,38 +288,38 @@ void PaletteItemBase::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
 	double deltaAngle = (a2 - a1) * 180 / M_PI;
 	//DebugDialog::debug(QString("original:%1 delta:%2").arg(originalAngle).arg(deltaAngle));
 	switch (m_viewID) {
-		case ViewLayer::BreadboardView:
-		case ViewLayer::PCBView:
-			{
-				double nearest = qRound((originalAngle + deltaAngle) / 45) * 45;
-				if (qAbs(originalAngle + deltaAngle - nearest) < 6) {
-					deltaAngle = nearest - originalAngle;
-					//DebugDialog::debug(QString("\tdelta angle %1").arg(deltaAngle));
-				}
-			}
-			break;
-		case ViewLayer::SchematicView:
-			{
-				double nearest = qRound((originalAngle + deltaAngle) / 90) * 90;
-				deltaAngle = nearest - originalAngle;
-			}
-			break;
-		default:
-			return;
+	case ViewLayer::BreadboardView:
+	case ViewLayer::PCBView:
+	{
+		double nearest = qRound((originalAngle + deltaAngle) / 45) * 45;
+		if (qAbs(originalAngle + deltaAngle - nearest) < 6) {
+			deltaAngle = nearest - originalAngle;
+			//DebugDialog::debug(QString("\tdelta angle %1").arg(deltaAngle));
+		}
+	}
+	break;
+	case ViewLayer::SchematicView:
+	{
+		double nearest = qRound((originalAngle + deltaAngle) / 90) * 90;
+		deltaAngle = nearest - originalAngle;
+	}
+	break;
+	default:
+		return;
 	}
 
 	ItemBase * chief = layerKinChief();
 	// restore viewGeometry to original angle
 	chief->getViewGeometry().setTransform(OriginalTransform);
-	foreach (ItemBase * itemBase, chief->layerKin()) {
+	Q_FOREACH (ItemBase * itemBase, chief->layerKin()) {
 		itemBase->getViewGeometry().setTransform(OriginalTransform);
 	}
 
 	//DebugDialog::debug(QString("rotating item %1 da:%2 oa:%3 %4").arg(QTime::currentTime().toString("HH:mm:ss.zzz")).arg(deltaAngle).arg(originalAngle).arg((long) this, 0, 16));
 	chief->rotateItem(deltaAngle, true);
 
-    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(chief);
-    if (infoGraphicsView) infoGraphicsView->updateRotation(chief);
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(chief);
+	if (infoGraphicsView != nullptr) infoGraphicsView->updateRotation(chief);
 }
 
 void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
@@ -330,7 +331,7 @@ void PaletteItemBase::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 
 	setInRotation(false);
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infoGraphicsView) {
+	if (infoGraphicsView != nullptr) {
 		// TODO: doesn't account for scaling
 		// see: http://www.gamedev.net/topic/441695-transform-matrix-decomposition/
 		double originalAngle = atan2(OriginalTransform.m12(), OriginalTransform.m11()) * 180 / M_PI;
@@ -348,22 +349,22 @@ bool PaletteItemBase::canFindConnectorsUnder() {
 void PaletteItemBase::findConnectorsUnder() {
 	if (!canFindConnectorsUnder()) return;
 
-	foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+	Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItems()) {
 		switch (connectorItem->connector()->connectorType()) {
-			case Connector::Female:
-			case Connector::Pad:
-				continue;
-			default:
-				break;
+		case Connector::Female:
+		case Connector::Pad:
+			continue;
+		default:
+			break;
 		}
 
-		connectorItem->findConnectorUnder(true, false, ConnectorItem::emptyConnectorItemList, false, NULL);
+		connectorItem->findConnectorUnder(true, false, ConnectorItem::emptyConnectorItemList, false, nullptr);
 	}
 }
 
 bool PaletteItemBase::collectFemaleConnectees(QSet<ItemBase *> & items) {
 	bool hasMale = false;
-	foreach (ConnectorItem * item, cachedConnectorItems()) {
+	Q_FOREACH (ConnectorItem * item, cachedConnectorItems()) {
 		if (item->connectorType() == Connector::Male) {
 			hasMale = true;
 			continue;
@@ -371,7 +372,7 @@ bool PaletteItemBase::collectFemaleConnectees(QSet<ItemBase *> & items) {
 
 		if (item->connectorType() != Connector::Female) continue;
 
-		foreach (ConnectorItem * toConnectorItem, item->connectedToItems()) {
+		Q_FOREACH (ConnectorItem * toConnectorItem, item->connectedToItems()) {
 			if (toConnectorItem->attachedToItemType() == ModelPart::Wire) continue;
 			if (!toConnectorItem->attachedTo()->isVisible()) continue;
 
@@ -383,8 +384,8 @@ bool PaletteItemBase::collectFemaleConnectees(QSet<ItemBase *> & items) {
 }
 
 void PaletteItemBase::collectWireConnectees(QSet<Wire *> & wires) {
-	foreach (ConnectorItem * item, cachedConnectorItems()) {
-		foreach (ConnectorItem * toConnectorItem, item->connectedToItems()) {
+	Q_FOREACH (ConnectorItem * item, cachedConnectorItems()) {
+		Q_FOREACH (ConnectorItem * toConnectorItem, item->connectedToItems()) {
 			if (toConnectorItem->attachedToItemType() == ModelPart::Wire) {
 				if (toConnectorItem->attachedTo()->isVisible()) {
 					wires.insert(qobject_cast<Wire *>(toConnectorItem->attachedTo()));
@@ -397,11 +398,11 @@ void PaletteItemBase::collectWireConnectees(QSet<Wire *> & wires) {
 bool PaletteItemBase::setUpImage(ModelPart * modelPart, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
 {
 	FSvgRenderer * renderer = ItemBase::setUpImage(modelPart, layerAttributes);
-	if (renderer == NULL) {
+	if (renderer == nullptr) {
 		return false;
 	}
 
-    cacheLoaded(layerAttributes);
+	cacheLoaded(layerAttributes);
 
 	m_canFlipVertical = modelPart->canFlipVertical(layerAttributes.viewID);
 	m_canFlipHorizontal = modelPart->canFlipHorizontal(layerAttributes.viewID);
@@ -413,8 +414,8 @@ bool PaletteItemBase::setUpImage(ModelPart * modelPart, const LayerHash & viewLa
 
 	//DebugDialog::debug(QString("setting layer %1 view:%2 z:%3").arg(modelPart->title()).arg(viewID).arg(this->z()) );
 	this->setZValue(this->z());
-    //DebugDialog::debug("loaded");
-    //DebugDialog::debug(layerAttributes.loaded());
+	//DebugDialog::debug("loaded");
+	//DebugDialog::debug(layerAttributes.loaded());
 	this->setSharedRendererEx(renderer);
 
 	m_svg = true;
@@ -435,44 +436,39 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 	clearConnectorItemCache();
 
 	if (m_viewID == ViewLayer::PCBView && ViewLayer::isNonCopperLayer(m_viewLayerID)) {
-		//DebugDialog::debug(QString("skip connectors: %1 vid:%2 vlid:%3")
-		//				   .arg(this->title())
-		//				   .arg(m_viewID) 
-		//				   .arg(m_viewLayerID)
-		//	);
 		// don't waste time
 		return;
 	}
 
-	foreach (Connector * connector, m_modelPart->connectors().values()) {
-		if (connector == NULL) continue;
+	Q_FOREACH (Connector * connector, m_modelPart->connectors().values()) {
+		if (connector == nullptr) continue;
 
 		//DebugDialog::debug(QString("id:%1 vid:%2 vlid:%3")
 		//				   .arg(connector->connectorSharedID())
-		//				   .arg(m_viewID) 
+		//				   .arg(m_viewID)
 		//				   .arg(m_viewLayerID)
 		//	);
 
 
 		SvgIdLayer * svgIdLayer = connector->fullPinInfo(m_viewID, m_viewLayerID);
-		if (svgIdLayer == NULL) {
+		if (svgIdLayer == nullptr) {
 			DebugDialog::debug(QString("svgidlayer fail %1 vid:%2 vlid:%3 %4")
-                .arg(connector->connectorSharedID())
-                .arg(m_viewID)
-                .arg(m_viewLayerID)
-                .arg(m_modelPart->path())
-                );
+			                   .arg(connector->connectorSharedID())
+			                   .arg(m_viewID)
+			                   .arg(m_viewLayerID)
+			                   .arg(m_modelPart->path())
+			                  );
 			continue;
 		}
 
 		bool result = renderer->setUpConnector(svgIdLayer, ignoreTerminalPoints, viewLayerPlacement());
 		if (!result) {
 			DebugDialog::debug(QString("setup connector fail %1 vid:%2 vlid:%3 %4")
-                .arg(connector->connectorSharedID())
-                .arg(m_viewID)
-                .arg(m_viewLayerID)
-                .arg(m_modelPart->path())
-                );
+			                   .arg(connector->connectorSharedID())
+			                   .arg(m_viewID)
+			                   .arg(m_viewLayerID)
+			                   .arg(m_modelPart->path())
+			                  );
 			continue;
 		}
 
@@ -483,7 +479,7 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 		connectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
 		connectorItem->setTerminalPoint(svgIdLayer->point(viewLayerPlacement()));
 		connectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
-        connectorItem->setIsPath(svgIdLayer->m_path);
+		connectorItem->setIsPath(svgIdLayer->m_path);
 		if (!svgIdLayer->m_legId.isEmpty()) {
 			m_hasRubberBandLeg = true;
 			connectorItem->setRubberBandLeg(QColor(svgIdLayer->m_legColor), svgIdLayer->m_legStrokeWidth, svgIdLayer->m_legLine);
@@ -493,18 +489,18 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 
 	}
 
-	foreach (SvgIdLayer * svgIdLayer, renderer->setUpNonConnectors(viewLayerPlacement())) {
-		if (svgIdLayer == NULL) continue;
+	Q_FOREACH (SvgIdLayer * svgIdLayer, renderer->setUpNonConnectors(viewLayerPlacement())) {
+		if (svgIdLayer == nullptr) continue;
 
-		NonConnectorItem * nonConnectorItem = new NonConnectorItem(this);
+		auto * nonConnectorItem = new NonConnectorItem(this);
 
 		//DebugDialog::debug(	QString("in layer %1 with z %2")
-			//.arg(ViewLayer::viewLayerNameFromID(m_viewLayerID))
-			//.arg(this->zValue()) );
+		//.arg(ViewLayer::viewLayerNameFromID(m_viewLayerID))
+		//.arg(this->zValue()) );
 
 		nonConnectorItem->setRect(svgIdLayer->rect(viewLayerPlacement()));
 		nonConnectorItem->setRadius(svgIdLayer->m_radius, svgIdLayer->m_strokeWidth);
-        nonConnectorItem->setIsPath(svgIdLayer->m_path);
+		nonConnectorItem->setIsPath(svgIdLayer->m_path);
 		//DebugDialog::debug(QString("terminal point %1 %2").arg(terminalPoint.x()).arg(terminalPoint.y()) );
 
 		delete svgIdLayer;
@@ -512,10 +508,10 @@ void PaletteItemBase::setUpConnectors(FSvgRenderer * renderer, bool ignoreTermin
 }
 
 void PaletteItemBase::connectedMoved(ConnectorItem * from, ConnectorItem * to,  QList<ConnectorItem *> & already) {
-    // not sure this is necessary any longer
-    return;
+	// not sure this is necessary any longer
+	return;
 
-    if (from->connectorType() != Connector::Female) return;
+	if (from->connectorType() != Connector::Female) return;
 
 	// female connectors really only operate in breadboard view
 	if (m_viewID != ViewLayer::BreadboardView) return;
@@ -548,12 +544,13 @@ void PaletteItemBase::hoverEnterEvent ( QGraphicsSceneHoverEvent * event ) {
 			CursorMaster::instance()->addCursor(this, cursor());
 
 			bool connected = false;
-			foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+			Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItems()) {
 				if (connectorItem->connectionsCount() > 0) {
 					connected = true;
 					break;
 				}
 			}
+			(void)connected;
 		}
 
 		checkFreeRotation(event->modifiers(), event->scenePos());
@@ -597,38 +594,38 @@ void PaletteItemBase::contextMenuEvent(QGraphicsSceneContextMenuEvent *event) {
 }
 
 
-LayerKinPaletteItem *PaletteItemBase::newLayerKinPaletteItem(PaletteItemBase * chief, ModelPart * modelPart, 
-															 const ViewGeometry & viewGeometry, long id,
-															 QMenu* itemMenu, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
+LayerKinPaletteItem *PaletteItemBase::newLayerKinPaletteItem(PaletteItemBase * chief, ModelPart * modelPart,
+        const ViewGeometry & viewGeometry, long id,
+        QMenu* itemMenu, const LayerHash & viewLayers, LayerAttributes & layerAttributes)
 {
-	LayerKinPaletteItem *lk = NULL;
-    if (layerAttributes.viewLayerID == ViewLayer::SchematicText) {
-        lk = new SchematicTextLayerKinPaletteItem(chief, modelPart, layerAttributes.viewID, viewGeometry, id, itemMenu);
-    }
-    else {
-        lk = new LayerKinPaletteItem(chief, modelPart, layerAttributes.viewID, viewGeometry, id, itemMenu);
-    }
+	LayerKinPaletteItem *lk = nullptr;
+	if (layerAttributes.viewLayerID == ViewLayer::SchematicText) {
+		lk = new SchematicTextLayerKinPaletteItem(chief, modelPart, layerAttributes.viewID, viewGeometry, id, itemMenu);
+	}
+	else {
+		lk = new LayerKinPaletteItem(chief, modelPart, layerAttributes.viewID, viewGeometry, id, itemMenu);
+	}
 	lk->initLKPI(layerAttributes, viewLayers);
 	return lk;
 }
 
-QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor) 
+QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<QString, QString> & svgHash, bool blackOnly, double dpi, double & factor)
 {
 	QString xmlName = ViewLayer::viewLayerXmlNameFromID(viewLayerID);
 	QString path = filename();
 
-    Qt::Orientations orientation = Qt::Vertical;
-    InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-    if (infoGraphicsView) {
-        orientation = infoGraphicsView->smdOrientation();
-    }
+	Qt::Orientations orientation = Qt::Vertical;
+	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infoGraphicsView != nullptr) {
+		orientation = infoGraphicsView->smdOrientation();
+	}
 
 	QDomDocument flipDoc;
-    getFlipDoc(modelPart(), path, viewLayerID, m_viewLayerPlacement, flipDoc, orientation);
-	
+	getFlipDoc(modelPart(), path, viewLayerID, m_viewLayerPlacement, flipDoc, orientation);
+
 	//DebugDialog::debug(QString("path: %1").arg(path));
 
-	QString svg = svgHash.value(path + xmlName, "");
+	QString svg = svgHash.value(path + xmlName + QString(QChar(m_viewLayerPlacement)), "");
 	if (!svg.isEmpty()) return svg;
 
 	SvgFileSplitter splitter;
@@ -638,7 +635,7 @@ QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<Q
 		result = splitter.split(path, xmlName);
 	}
 	else {
-		QString f = flipDoc.toString(); 
+		QString f = flipDoc.toString();
 		result = splitter.splitString(f, xmlName);
 	}
 
@@ -647,7 +644,7 @@ QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<Q
 	}
 
 	if (hasRubberBandLeg()) {
-		foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+		Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItems()) {
 			if (!connectorItem->hasRubberBandLeg()) continue;
 
 			splitter.gReplace(connectorItem->legID(m_viewID, m_viewLayerID));
@@ -659,7 +656,7 @@ QString PaletteItemBase::retrieveSvg(ViewLayer::ViewLayerID viewLayerID, QHash<Q
 		return "";
 	}
 	svg = splitter.elementString(xmlName);
-	svgHash.insert(path + xmlName, svg);
+	svgHash.insert(path + xmlName + QString(QChar(m_viewLayerPlacement)), svg);
 	return svg;
 }
 
@@ -671,41 +668,58 @@ bool PaletteItemBase::canEditPart() {
 
 bool PaletteItemBase::collectExtraInfo(QWidget * parent, const QString & family, const QString & prop, const QString & value, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget, bool & hide)
 {
-	if (prop.compare(ModelPartShared::PartNumberPropertyName, Qt::CaseInsensitive) == 0) {
-		returnProp = TranslatedPropertyNames.value(prop);
-
-		QLineEdit * lineEdit = new QLineEdit();
-		lineEdit->setEnabled(swappingEnabled);
-		QString current = m_modelPart->localProp(ModelPartShared::PartNumberPropertyName).toString();
-		lineEdit->setText(current);
-		connect(lineEdit, SIGNAL(editingFinished()), this, SLOT(partPropertyEntry()));	
-		lineEdit->setObjectName("infoViewLineEdit");		
-		returnWidget = lineEdit;	
-		returnValue = current;
-		return true;
+	for (auto&& propertyName : {ModelPartShared::MNPropertyName, ModelPartShared::MPNPropertyName, ModelPartShared::PartNumberPropertyName}) {
+		if (collectExtraInfoPartNumber(propertyName, prop, swappingEnabled, returnProp, returnValue, returnWidget)) return true;
 	}
-
 	return ItemBase::collectExtraInfo(parent, family, prop, value, swappingEnabled, returnProp, returnValue, returnWidget, hide);
 }
 
-void PaletteItemBase::setProp(const QString & prop, const QString & value) 
-{	
-	if (prop.compare(ModelPartShared::PartNumberPropertyName) == 0) {
-		modelPart()->setLocalProp(ModelPartShared::PartNumberPropertyName, value);
-		if (m_partLabel) m_partLabel->displayTextsIf();
-		return;
-	}
+bool PaletteItemBase::collectExtraInfoPartNumber(const QString & propertyName, const QString & prop, bool swappingEnabled, QString & returnProp, QString & returnValue, QWidget * & returnWidget)
+{
+	if (prop.compare(propertyName, Qt::CaseInsensitive) == 0) {
+		returnProp = TranslatedPropertyNames.value(prop);
 
+		auto * lineEdit = new QLineEdit();
+		QRegularExpressionValidator *validator = new QRegularExpressionValidator(QRegularExpression("([^ \t].*[^ \t]|[^ \t])"), this);
+		lineEdit->setValidator(validator);
+		lineEdit->setEnabled(swappingEnabled);
+		QString current = m_modelPart->localProp(propertyName).toString();
+		lineEdit->setText(current);
+		lineEdit->setProperty("property name for entry", propertyName);
+		connect(lineEdit, SIGNAL(editingFinished()), this, SLOT(partPropertyEntry()));
+		lineEdit->setObjectName("infoViewLineEdit");
+		returnWidget = lineEdit;
+		returnValue = current;
+		return true;
+	}
+	return false;
+}
+
+void PaletteItemBase::setProp(const QString & prop, const QString & value)
+{
+	for (auto&& propertyName : {ModelPartShared::MNPropertyName, ModelPartShared::MPNPropertyName, ModelPartShared::PartNumberPropertyName}) {
+		setLocalProp(prop, value, propertyName);
+	}
 	ItemBase::setProp(prop, value);
 }
 
+void PaletteItemBase::setLocalProp(const QString & prop, const QString & value, const QString & propertyName)
+{
+	if (prop.compare(propertyName) == 0) {
+		modelPart()->setLocalProp(propertyName, value);
+		if (m_partLabel != nullptr) m_partLabel->displayTextsIf();
+		return;
+	}
+}
+
 void PaletteItemBase::partPropertyEntry() {
-	QLineEdit * lineEdit = qobject_cast<QLineEdit *>(sender());
-	if (lineEdit == NULL) return;
+	auto * lineEdit = qobject_cast<QLineEdit *>(sender());
+	if (lineEdit == nullptr) return;
 
 	InfoGraphicsView * infoGraphicsView = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infoGraphicsView != NULL) {
-		infoGraphicsView->setProp(this, ModelPartShared::PartNumberPropertyName, "", m_modelPart->localProp(ModelPartShared::PartNumberPropertyName).toString(), lineEdit->text(), true);
+	if (infoGraphicsView != nullptr) {
+		QString propertyName = lineEdit->property("property name for entry").toString();
+		infoGraphicsView->setProp(this, propertyName, "", m_modelPart->localProp(propertyName).toString(), lineEdit->text(), true);
 	}
 }
 
@@ -713,8 +727,8 @@ void PaletteItemBase::partPropertyEntry() {
 const QCursor * PaletteItemBase::getCursor(Qt::KeyboardModifiers modifiers)
 {
 	if (hasRubberBandLeg()) {
-		if ((modifiers & altOrMetaModifier())) {
-			foreach (ConnectorItem * connectorItem, cachedConnectorItems()) {
+		if ((modifiers & altOrMetaModifier()) != 0u) {
+			Q_FOREACH (ConnectorItem * connectorItem, cachedConnectorItems()) {
 				if (connectorItem->connectionsCount() > 0) {
 					return CursorMaster::RubberbandCursor;
 				}
@@ -736,7 +750,7 @@ bool PaletteItemBase::freeRotationAllowed(Qt::KeyboardModifiers modifiers) {
 }
 
 bool PaletteItemBase::freeRotationAllowed() {
-    if (viewID() == ViewLayer::SchematicView) return false;
+	if (viewID() == ViewLayer::SchematicView) return false;
 
 	return rotation45Allowed();
 }
@@ -752,7 +766,7 @@ bool PaletteItemBase::inRotationLocation(QPointF scenePos, Qt::KeyboardModifiers
 	polygon.append(mapToScene(r.topRight()));
 	polygon.append(mapToScene(r.bottomRight()));
 	polygon.append(mapToScene(r.bottomLeft()));
-	foreach (QPointF p, polygon) {
+	Q_FOREACH (QPointF p, polygon) {
 		double dsqd = GraphicsUtils::distanceSqd(p, scenePos);
 		if (dsqd < 9) {
 			returnPoint = p;
@@ -795,7 +809,7 @@ QString PaletteItemBase::normalizeSvg(QString & svg, ViewLayer::ViewLayerID view
 
 void PaletteItemBase::setInitialTransform(const QTransform &matrix)
 {
-    setTransform(matrix);
+	setTransform(matrix);
 }
 
 void PaletteItemBase::cacheLoaded(const LayerAttributes &)
@@ -815,4 +829,3 @@ void PaletteItemBase::setPos(double x, double y) {
 
 
 */
-
