@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,24 +16,15 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6912 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
-
 ********************************************************************/
 
 #include "partlabel.h"
 #include "../items/itembase.h"
-#include "../viewgeometry.h"
-#include "../debugdialog.h"
 #include "../sketch/infographicsview.h"
 #include "../model/modelpart.h"
 #include "../utils/graphicsutils.h"
 #include "../utils/textutils.h"
 #include "../installedfonts.h"
-#include "../fsvgrenderer.h"
 
 #include <QGraphicsScene>
 #include <QMenu>
@@ -48,7 +39,7 @@ $Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
 //		** select a part, highlight its label; click a label, highlight its part
 //		** viewinfo update when selected
 //		** viewinfo for wires
-//		** graphics (esp. drag area vs. edit area) 
+//		** graphics (esp. drag area vs. edit area)
 //		** html info box needs to update when view switches
 //		** undo delete text
 //		** undo change text
@@ -62,9 +53,9 @@ $Date: 2013-03-09 08:18:59 +0100 (Sa, 09. Mrz 2013) $
 //		** delete owner: delete label
 //		** make label single-line (ignore enter key)
 
-//		** rotate/flip 
+//		** rotate/flip
 //		** undo rotate/flip
-//		format: bold, italic, size (small normal large huge), color?, 
+//		format: bold, italic, size (small normal large huge), color?,
 //		undo format
 //		heads-up controls
 
@@ -92,7 +83,7 @@ enum PartLabelAction {
 	PartLabelFlipHorizontal,
 	PartLabelFlipVertical,
 	PartLabelEdit,
-    PartLabelFontSizeTiny,
+	PartLabelFontSizeTiny,
 	PartLabelFontSizeSmall,
 	PartLabelFontSizeMedium,
 	PartLabelFontSizeLarge,
@@ -104,36 +95,38 @@ enum PartLabelAction {
 
 static QMultiHash<long, PartLabel *> AllPartLabels;
 static const QString LabelTextKey = "";
-static const double InactiveOpacity = 0.4;
+static constexpr double InactiveOpacity = 0.4;
 
 ///////////////////////////////////////////
 
-PartLabel::PartLabel(ItemBase * owner, QGraphicsItem * parent)
-	: QGraphicsSvgItem(parent)
-{
-	m_renderer = NULL;
-	m_owner = owner;
-	m_spaceBarWasPressed = false;
-	m_text = m_displayText = "";
+PartLabel::PartLabel(ItemBase * owner, QWidget *parentWidget, QGraphicsItem * parent)
+	: QGraphicsSvgItem(parent),
+	  m_owner(owner),
+	  m_parentWidget(parentWidget)
 
-	m_inactive = m_hidden = m_initialized = false;
+{
 	m_displayKeys.append(LabelTextKey);
 	if (m_owner->hasPartNumberProperty() && m_owner->viewID() != ViewLayer::PCBView) {
-		m_displayKeys.append(ModelPartShared::PartNumberPropertyName);
+		for (auto&& propertyName : {ModelPartShared::MNPropertyName, ModelPartShared::MPNPropertyName, ModelPartShared::PartNumberPropertyName}) {
+			m_displayKeys.append(propertyName);
+		}
 	}
 
 	setFlag(QGraphicsItem::ItemIsSelectable, false);
 	setFlag(QGraphicsItem::ItemIsMovable, false);					// don't move this in the standard QGraphicsItem way
 	setVisible(false);
-	m_viewLayerID = ViewLayer::UnknownLayer;
 	setAcceptHoverEvents(true);
 	AllPartLabels.insert(m_owner->id(), this);
+	// No translation needed, the menu title is only shown if the menu is added
+	// to a menu bar
+	//	m_menu = new QMenu(QObject::tr("PartLabel"), m_parentWidget);
+	m_menu = new QMenu("PartLabel", m_parentWidget);
 }
 
-PartLabel::~PartLabel() 
+PartLabel::~PartLabel()
 {
 	AllPartLabels.remove(m_owner->id(), this);
-	if (m_owner) {
+	if (m_owner != nullptr) {
 		m_owner->clearPartLabel();
 	}
 }
@@ -142,13 +135,13 @@ void PartLabel::showLabel(bool showIt, ViewLayer * viewLayer) {
 	if (showIt == this->isVisible()) return;
 
 	if (showIt && !m_initialized) {
-		if (m_owner == NULL) return;
-		if (m_owner->scene() == NULL) return;
+		if (m_owner == nullptr) return;
+		if (m_owner->scene() == nullptr) return;
 
 		bool flipped = (viewLayer->viewLayerID() == ViewLayer::Silkscreen0Label);
 
 		if (m_owner->viewID() != ViewLayer::PCBView) {
-			foreach (QString dk, m_owner->modelPart()->displayKeys()) {
+			Q_FOREACH (QString dk, m_owner->modelPart()->displayKeys()) {
 				if (!m_displayKeys.contains(dk)) {
 					m_displayKeys.append(dk);
 				}
@@ -164,11 +157,14 @@ void PartLabel::showLabel(bool showIt, ViewLayer * viewLayer) {
 
 		QRectF obr = m_owner->boundingRect();
 		QRectF tbr = QGraphicsSvgItem::boundingRect();
-		QPointF initial = (flipped) 
-			? m_owner->pos() + QPointF(-tbr.width(), -tbr.height())
-			: m_owner->pos() + QPointF(obr.width(), -tbr.height());
-		this->setPos(initial);
-		m_offset = initial - m_owner->pos();
+		QPointF initial = (flipped)
+						  ? m_owner->pos() + QPointF(-tbr.width(), -tbr.height())
+						  : m_owner->pos() + QPointF(obr.width(), -tbr.height());
+		if (!m_initializedPos) {
+			this->setPos(initial);
+			m_offset = initial - m_owner->pos();
+			m_initializedPos = true;
+		}
 		if (flipped) {
 			transformLabel(QTransform().scale(-1,1));
 		}
@@ -183,7 +179,7 @@ QPainterPath PartLabel::shape() const
 	QRectF t = boundingRect();
 	QPainterPath path;
 	path.addRect(t);
-    return path;
+	return path;
 }
 
 void PartLabel::mousePressEvent(QGraphicsSceneMouseEvent *event)
@@ -194,7 +190,7 @@ void PartLabel::mousePressEvent(QGraphicsSceneMouseEvent *event)
 	}
 
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infographics != NULL && infographics->spaceBarIsPressed()) {
+	if ((infographics != nullptr) && infographics->spaceBarIsPressed()) {
 		m_spaceBarWasPressed = true;
 		event->ignore();
 		return;
@@ -255,7 +251,7 @@ void PartLabel::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 }
 
 
-void PartLabel::setPlainText(const QString & text) 
+void PartLabel::setPlainText(const QString & text)
 {
 	m_text = text;
 	displayTexts();
@@ -272,7 +268,7 @@ void PartLabel::displayTextsIf() {
 void PartLabel::displayTexts() {
 	QStringList texts;
 
-	foreach (QString key, m_displayKeys) {
+	Q_FOREACH (QString key, m_displayKeys) {
 		QString t;
 		if (key.compare(LabelTextKey) == 0) {
 			t = m_text;
@@ -293,10 +289,6 @@ void PartLabel::displayTexts() {
 	resetSvg();
 }
 
-bool PartLabel::initialized() {
-	return m_initialized;
-}
-
 void PartLabel::ownerMoved(QPointF newPos) {
 	this->setPos(m_offset + newPos);
 }
@@ -323,20 +315,18 @@ void PartLabel::setHiddenOrInactive() {
 	update();
 }
 
-
-ViewLayer::ViewLayerID PartLabel::viewLayerID() {
-	return m_viewLayerID;
+bool PartLabel::isFlipped(ViewLayer::ViewLayerID viewLayerID) {
+	bool flipped = (viewLayerID == ViewLayer::Silkscreen0Label);
+	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
+	if (infographics != nullptr) {
+		if (infographics->viewFromBelow()) {
+			flipped = !flipped;
+		}
+	}
+	return flipped;
 }
 
-bool PartLabel::hidden() {
-	return m_hidden;
-}
-
-bool PartLabel::inactive() {
-	return m_inactive;
-}
-
-void PartLabel::saveInstance(QXmlStreamWriter & streamWriter) {
+void PartLabel::saveInstance(QXmlStreamWriter & streamWriter, bool flipAware) {
 	if (!m_initialized) return;
 
 	streamWriter.writeStartElement("titleGeometry");
@@ -348,8 +338,14 @@ void PartLabel::saveInstance(QXmlStreamWriter & streamWriter) {
 	streamWriter.writeAttribute("yOffset", QString::number(m_offset.y()));
 	streamWriter.writeAttribute("textColor", m_color.name());
 	streamWriter.writeAttribute("fontSize", QString::number(m_font.pointSizeF()));
-	GraphicsUtils::saveTransform(streamWriter, transform());
-	foreach (QString key, m_displayKeys) {
+	QTransform transformation = transform();
+	if (flipAware && isFlipped(m_viewLayerID)) {
+		transformLabel(QTransform().scale(-1,1));
+		transformation = transform();
+		transformLabel(QTransform().scale(-1,1));
+	}
+	GraphicsUtils::saveTransform(streamWriter, transformation);
+	Q_FOREACH (QString key, m_displayKeys) {
 		streamWriter.writeStartElement("displayKey");
 		streamWriter.writeAttribute("key", key);
 		streamWriter.writeEndElement();
@@ -357,7 +353,19 @@ void PartLabel::saveInstance(QXmlStreamWriter & streamWriter) {
 	streamWriter.writeEndElement();
 }
 
-void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID viewLayerID) 
+void PartLabel::getLabelGeometry(QDomElement & labelGeometry) {
+	if (!m_initialized) return;
+	QByteArray data;
+	QXmlStreamWriter streamWriter(&data);
+	saveInstance(streamWriter, true);
+
+	QDomDocument temporary;
+	if (temporary.setContent(data)) {
+		labelGeometry = temporary.documentElement();
+	}
+}
+
+void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID viewLayerID, bool flipAware)
 {
 	m_viewLayerID = viewLayerID;
 	m_initialized = true;
@@ -387,8 +395,8 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 	double fs = labelGeometry.attribute("fontSize").toDouble(&ok);
 	if (!ok) {
 		InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
-		if (infographics != NULL) {
-            fs = infographics->getLabelFontSizeMedium();
+		if (infographics != nullptr) {
+			fs = infographics->getLabelFontSizeMedium();
 			ok = true;
 		}
 	}
@@ -405,7 +413,11 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 
 	if (m_displayKeys.length() == 0) {
 		m_displayKeys.append(LabelTextKey);
-		if (m_owner->hasPartNumberProperty()) m_displayKeys.append(ModelPartShared::PartNumberPropertyName);
+		if (m_owner->hasPartNumberProperty()) {
+			for (auto&& propertyName : {ModelPartShared::MNPropertyName, ModelPartShared::MPNPropertyName, ModelPartShared::PartNumberPropertyName}) {
+				m_displayKeys.append(propertyName);
+			}
+		}
 	}
 
 	displayTexts();
@@ -414,70 +426,79 @@ void PartLabel::restoreLabel(QDomElement & labelGeometry, ViewLayer::ViewLayerID
 	if (GraphicsUtils::loadTransform(labelGeometry.firstChildElement("transform"), t)) {
 		setTransform(t);
 	}
+
+	if (flipAware && isFlipped(viewLayerID)) {
+		transformLabel(QTransform().scale(-1,1));
+	}
 }
 
-void PartLabel::moveLabel(QPointF newPos, QPointF newOffset) 
+void PartLabel::moveLabel(QPointF newPos, QPointF newOffset)
 {
 	this->setPos(newPos);
+	m_initializedPos = true;
 	m_offset = newOffset;
 }
 
+QPointF PartLabel::getOffset()
+{
+	return m_offset;
+}
+
 ItemBase * PartLabel::owner() {
 	return m_owner;
 }
 
-void PartLabel::initMenu() 
+void PartLabel::initMenu()
 {
 	// todo: make this a static var?
-
-    QAction *editAct = m_menu.addAction(tr("Edit"));
+	QAction *editAct = m_menu->addAction(tr("Edit"));
 	editAct->setData(QVariant(PartLabelEdit));
 	editAct->setStatusTip(tr("Edit label text"));
 
-    QAction *hideAct = m_menu.addAction(tr("Hide"));
+	QAction *hideAct = m_menu->addAction(tr("Hide"));
 	hideAct->setData(QVariant(PartLabelHide));
 	hideAct->setStatusTip(tr("Hide part label"));
 
-	m_menu.addSeparator();
+	m_menu->addSeparator();
 
-	QMenu * dvmenu = m_menu.addMenu(tr("Display Values"));
-	QMenu * rlmenu = m_menu.addMenu(tr("Flip/Rotate"));
-	QMenu * fsmenu = m_menu.addMenu(tr("Font Size"));
+	QMenu * dvmenu = m_menu->addMenu(tr("Display Values"));
+	QMenu * rlmenu = m_menu->addMenu(tr("Flip/Rotate"));
+	QMenu * fsmenu = m_menu->addMenu(tr("Font Size"));
 
-	bool include45 = (m_owner != NULL) && (m_owner->viewID() == ViewLayer::PCBView);
+	bool include45 = ((m_owner) != nullptr) && (m_owner->viewID() == ViewLayer::PCBView);
 
 	if (include45) {
-		QAction *rotate45cwAct = rlmenu->addAction(tr("Rotate 45\x00B0 Clockwise"));
+		QAction *rotate45cwAct = rlmenu->addAction(tr("Rotate 45° Clockwise"));
 		rotate45cwAct->setData(QVariant(PartLabelRotate45CW));
 		rotate45cwAct->setStatusTip(tr("Rotate the label by 45 degrees clockwise"));
 	}
 
-    QAction *rotate90cwAct = rlmenu->addAction(tr("Rotate 90\x00B0 Clockwise"));
+	QAction *rotate90cwAct = rlmenu->addAction(tr("Rotate 90° Clockwise"));
 	rotate90cwAct->setData(QVariant(PartLabelRotate90CW));
 	rotate90cwAct->setStatusTip(tr("Rotate the label by 90 degrees clockwise"));
 
 	if (include45) {
-		QAction *rotate135cwAct = rlmenu->addAction(tr("Rotate 135\x00B0 Clockwise"));
+		QAction *rotate135cwAct = rlmenu->addAction(tr("Rotate 135° Clockwise"));
 		rotate135cwAct->setData(QVariant(PartLabelRotate135CW));
 		rotate135cwAct->setStatusTip(tr("Rotate the label by 135 degrees clockwise"));
 	}
 
- 	QAction *rotate180Act = rlmenu->addAction(tr("Rotate 180\x00B0"));
+	QAction *rotate180Act = rlmenu->addAction(tr("Rotate 180°"));
 	rotate180Act->setData(QVariant(PartLabelRotate180));
 	rotate180Act->setStatusTip(tr("Rotate the label by 180 degrees"));
-  
+
 	if (include45) {
-		QAction *rotate135ccwAct = rlmenu->addAction(tr("Rotate 135\x00B0 Counter Clockwise"));
+		QAction *rotate135ccwAct = rlmenu->addAction(tr("Rotate 135° Counter Clockwise"));
 		rotate135ccwAct->setData(QVariant(PartLabelRotate135CCW));
 		rotate135ccwAct->setStatusTip(tr("Rotate the label by 135 degrees counter clockwise"));
 	}
 
-	QAction *rotate90ccwAct = rlmenu->addAction(tr("Rotate 90\x00B0 Counter Clockwise"));
+	QAction *rotate90ccwAct = rlmenu->addAction(tr("Rotate 90° Counter Clockwise"));
 	rotate90ccwAct->setData(QVariant(PartLabelRotate90CCW));
 	rotate90ccwAct->setStatusTip(tr("Rotate current selection 90 degrees counter clockwise"));
-		
+
 	if (include45) {
-		QAction *rotate45ccwAct = rlmenu->addAction(tr("Rotate 45\x00B0 Counter Clockwise"));
+		QAction *rotate45ccwAct = rlmenu->addAction(tr("Rotate 45° Counter Clockwise"));
 		rotate45ccwAct->setData(QVariant(PartLabelRotate45CCW));
 		rotate45ccwAct->setStatusTip(tr("Rotate the label by 45 degrees counter clockwise"));
 	}
@@ -490,31 +511,31 @@ void PartLabel::initMenu()
 	flipVerticalAct->setData(QVariant(PartLabelFlipVertical));
 	flipVerticalAct->setStatusTip(tr("Flip label vertically"));
 
-    m_tinyAct = fsmenu->addAction(tr("Tiny"));
-        m_tinyAct->setData(QVariant(PartLabelFontSizeTiny));
-        m_tinyAct->setStatusTip(tr("Set font size to tiny"));
-        m_tinyAct->setCheckable(true);
-        m_tinyAct->setChecked(false);
+	m_tinyAct = fsmenu->addAction(tr("Tiny"));
+	m_tinyAct->setData(QVariant(PartLabelFontSizeTiny));
+	m_tinyAct->setStatusTip(tr("Set font size to tiny"));
+	m_tinyAct->setCheckable(true);
+	m_tinyAct->setChecked(false);
 
-    m_smallAct = fsmenu->addAction(tr("Small"));
+	m_smallAct = fsmenu->addAction(tr("Small"));
 	m_smallAct->setData(QVariant(PartLabelFontSizeSmall));
 	m_smallAct->setStatusTip(tr("Set font size to small"));
 	m_smallAct->setCheckable(true);
 	m_smallAct->setChecked(false);
 
-    m_mediumAct = fsmenu->addAction(tr("Medium"));
+	m_mediumAct = fsmenu->addAction(tr("Medium"));
 	m_mediumAct->setData(QVariant(PartLabelFontSizeMedium));
 	m_mediumAct->setStatusTip(tr("Set font size to medium"));
 	m_mediumAct->setCheckable(true);
 	m_mediumAct->setChecked(false);
 
-    m_largeAct = fsmenu->addAction(tr("Large"));
+	m_largeAct = fsmenu->addAction(tr("Large"));
 	m_largeAct->setData(QVariant(PartLabelFontSizeLarge));
 	m_largeAct->setStatusTip(tr("Set font size to large"));
 	m_largeAct->setCheckable(true);
 	m_largeAct->setChecked(false);
 
-    m_labelAct = dvmenu->addAction(tr("Label text"));
+	m_labelAct = dvmenu->addAction(tr("Label text"));
 	m_labelAct->setData(QVariant(PartLabelDisplayLabelText));
 	m_labelAct->setCheckable(true);
 	m_labelAct->setChecked(true);
@@ -523,7 +544,7 @@ void PartLabel::initMenu()
 	dvmenu->addSeparator();
 
 	QHash<QString,QString> properties = m_owner->modelPart()->properties();
-	foreach (QString key, properties.keys()) {
+	Q_FOREACH (QString key, properties.keys()) {
 		QString translatedName = ItemBase::translatePropertyName(key);
 		QAction * action = dvmenu->addAction(translatedName);
 		action->setData(QVariant(key));
@@ -543,7 +564,7 @@ void PartLabel::rotateFlipLabel(double degrees, Qt::Orientations orientation) {
 		if (orientation == Qt::Vertical) {
 			xScale = 1;
 			yScale = -1;
-		} 
+		}
 		else if(orientation == Qt::Horizontal) {
 			xScale = -1;
 			yScale = 1;
@@ -554,7 +575,7 @@ void PartLabel::rotateFlipLabel(double degrees, Qt::Orientations orientation) {
 	}
 }
 
-void PartLabel::transformLabel(QTransform currTransf) 
+void PartLabel::transformLabel(QTransform currTransf)
 {
 	QRectF rect = this->boundingRect();
 	double x = rect.width() / 2.0;
@@ -565,7 +586,7 @@ void PartLabel::transformLabel(QTransform currTransf)
 
 void PartLabel::setUpText() {
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infographics != NULL) {
+	if (infographics != nullptr) {
 		infographics->getLabelFont(m_font, m_color, m_owner);
 	}
 }
@@ -573,18 +594,18 @@ void PartLabel::setUpText() {
 QVariant PartLabel::itemChange(QGraphicsItem::GraphicsItemChange change, const QVariant & value)
 {
 	switch (change) {
-		case QGraphicsItem::ItemSceneHasChanged:
-			if (this->scene()) {
-			}
-			break;
-		default:
-			break;
+	case QGraphicsItem::ItemSceneHasChanged:
+		if (this->scene() != nullptr) {
+		}
+		break;
+	default:
+		break;
 	}
 
 	return QGraphicsSvgItem::itemChange(change, value);
 }
 
-void PartLabel::ownerSelected(bool selected) 
+void PartLabel::ownerSelected(bool selected)
 {
 	bool hide = !selected;
 	if (m_hidden || m_inactive) {
@@ -601,29 +622,29 @@ void PartLabel::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 		return;
 	}
 
-	if (m_menu.isEmpty()) {
+	if (m_menu->isEmpty()) {
 		initMenu();
 	}
 
 	m_labelAct->setChecked(m_displayKeys.contains(LabelTextKey));
-	foreach (QAction * displayAct, m_displayActs) {
+	Q_FOREACH (QAction * displayAct, m_displayActs) {
 		QString data = displayAct->data().toString();
 		displayAct->setChecked(m_displayKeys.contains(data));
 	}
 
-    m_tinyAct->setChecked(false);
+	m_tinyAct->setChecked(false);
 	m_smallAct->setChecked(false);
 	m_mediumAct->setChecked(false);
 	m_largeAct->setChecked(false);
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infographics != NULL) {
+	if (infographics != nullptr) {
 		int fs = m_font.pointSize();
 		if (fs == infographics->getLabelFontSizeTiny()) {
 			m_tinyAct->setChecked(true);
 		}
 		else if (fs == infographics->getLabelFontSizeSmall()) {
-            m_smallAct->setChecked(true);
-        }
+			m_smallAct->setChecked(true);
+		}
 		else if (fs == infographics->getLabelFontSizeMedium()) {
 			m_mediumAct->setChecked(true);
 		}
@@ -631,78 +652,78 @@ void PartLabel::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 			m_largeAct->setChecked(true);
 		}
 	}
-    
-	QAction *selectedAction = m_menu.exec(event->screenPos());
-	if (selectedAction == NULL) return;
+
+	QAction *selectedAction = m_menu->exec(event->screenPos());
+	if (selectedAction == nullptr) return;
 
 	PartLabelAction action = (PartLabelAction) selectedAction->data().toInt();
 	switch (action) {
-		case PartLabelRotate45CW:
-		case PartLabelRotate45CCW:
-		case PartLabelRotate90CW:
-		case PartLabelRotate90CCW:
-		case PartLabelRotate135CW:
-		case PartLabelRotate135CCW:
-		case PartLabelRotate180:
-		case PartLabelFlipHorizontal:
-		case PartLabelFlipVertical:
-			rotateFlip(action);
-			break;
-		case PartLabelEdit:
-			partLabelEdit();
-			break;
-		case PartLabelHide:
-			partLabelHide();
-			break;
-                case PartLabelFontSizeTiny:
-		case PartLabelFontSizeSmall:
-		case PartLabelFontSizeMedium:
-		case PartLabelFontSizeLarge:
-			setFontSize(action);
-			resetSvg();
-			break;
-		case PartLabelDisplayLabelText:
-			setLabelDisplay(LabelTextKey);
-			break;
-		default:
-			setLabelDisplay(selectedAction->data().toString());
-			break;
+	case PartLabelRotate45CW:
+	case PartLabelRotate45CCW:
+	case PartLabelRotate90CW:
+	case PartLabelRotate90CCW:
+	case PartLabelRotate135CW:
+	case PartLabelRotate135CCW:
+	case PartLabelRotate180:
+	case PartLabelFlipHorizontal:
+	case PartLabelFlipVertical:
+		rotateFlip(action);
+		break;
+	case PartLabelEdit:
+		partLabelEdit();
+		break;
+	case PartLabelHide:
+		partLabelHide();
+		break;
+	case PartLabelFontSizeTiny:
+	case PartLabelFontSizeSmall:
+	case PartLabelFontSizeMedium:
+	case PartLabelFontSizeLarge:
+		setFontSize(action);
+		resetSvg();
+		break;
+	case PartLabelDisplayLabelText:
+		setLabelDisplay(LabelTextKey);
+		break;
+	default:
+		setLabelDisplay(selectedAction->data().toString());
+		break;
 	}
 }
 
 void PartLabel::rotateFlip(int action) {
 	double degrees = 0;
-	Qt::Orientations orientation = 0;
+	Qt::Orientations orientation = QFlags<Qt::Orientation>();
 	switch (action) {
-		case PartLabelRotate45CW:
-			degrees = 45;
-			break;
-		case PartLabelRotate90CW:
-			degrees = 90;
-			break;
-		case PartLabelRotate135CW:
-			degrees = 135;
-			break;
-		case PartLabelRotate90CCW:
-			degrees = 270;
-			break;
-		case PartLabelRotate135CCW:
-			degrees = 225;
-			break;
-		case PartLabelRotate180:
-			degrees = 180;
-			break;
-		case PartLabelRotate45CCW:
-			degrees = 315;
-			break;
-		case PartLabelFlipHorizontal:
-			orientation = Qt::Horizontal;
-			break;
-		case PartLabelFlipVertical:
-			orientation = Qt::Vertical;
-			break;
-		default:
-			break;
+	case PartLabelRotate45CW:
+		degrees = 45;
+		break;
+	case PartLabelRotate90CW:
+		degrees = 90;
+		break;
+	case PartLabelRotate135CW:
+		degrees = 135;
+		break;
+	case PartLabelRotate90CCW:
+		degrees = 270;
+		break;
+	case PartLabelRotate135CCW:
+		degrees = 225;
+		break;
+	case PartLabelRotate180:
+		degrees = 180;
+		break;
+	case PartLabelRotate45CCW:
+		degrees = 315;
+		break;
+	case PartLabelFlipHorizontal:
+		orientation = Qt::Horizontal;
+		break;
+	case PartLabelFlipVertical:
+		orientation = Qt::Vertical;
+		break;
+	default:
+		break;
 	}
 
 	m_owner->rotateFlipPartLabel(degrees, orientation);
@@ -719,24 +740,24 @@ void PartLabel::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event) {
 	partLabelEdit();
 }
 
-void PartLabel::partLabelHide() 
+void PartLabel::partLabelHide()
 {
 	m_owner->hidePartLabel();
 }
 
-void PartLabel::partLabelEdit() 
+void PartLabel::partLabelEdit()
 {
 	bool ok;
 	QString oldText = m_text;
-    QString text = QInputDialog::getText((QGraphicsView *) this->scene()->parent(), tr("Set label for %1").arg(m_owner->title()),
-                                          tr("Label text:"), QLineEdit::Normal, oldText, &ok);
+	QString text = QInputDialog::getText((QGraphicsView *) this->scene()->parent(), tr("Set label for %1").arg(m_owner->title()),
+										 tr("Label text:"), QLineEdit::Normal, oldText, &ok);
 	if (ok && (oldText.compare(text) != 0)) {
-		if (m_owner) {
+		if (m_owner != nullptr) {
 			m_owner->partLabelChanged(text);
-			foreach (PartLabel * p, AllPartLabels.values(m_owner->id())) {
+			Q_FOREACH (PartLabel * p, AllPartLabels.values(m_owner->id())) {
 				p->setPlainText(text);
 			}
-		}	
+		}
 	}
 }
 
@@ -751,9 +772,9 @@ void PartLabel::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 
 	if (m_owner->isSelected()) {
 		GraphicsUtils::qt_graphicsItem_highlightSelected(painter, option, boundingRect(), shape());
-    }
+	}
 
-    QGraphicsSvgItem::paint(painter, option, widget);
+	QGraphicsSvgItem::paint(painter, option, widget);
 
 	if (m_inactive) {
 		painter->restore();
@@ -762,31 +783,31 @@ void PartLabel::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
 
 void PartLabel::setFontSize(int action) {
 	InfoGraphicsView *infographics = InfoGraphicsView::getInfoGraphicsView(this);
-	if (infographics == NULL) return;
+	if (infographics == nullptr) return;
 
 	double fs = 0;
 	switch (action) {
-        case PartLabelFontSizeTiny:
-            fs = infographics->getLabelFontSizeTiny();
-            break;
-		case PartLabelFontSizeSmall:
-			fs = infographics->getLabelFontSizeSmall();
-			break;
-		case PartLabelFontSizeMedium:
-			fs = infographics->getLabelFontSizeMedium();
-			break;
-		case PartLabelFontSizeLarge:
-			fs = infographics->getLabelFontSizeLarge();
-			break;
-		default:
-			return;
+	case PartLabelFontSizeTiny:
+		fs = infographics->getLabelFontSizeTiny();
+		break;
+	case PartLabelFontSizeSmall:
+		fs = infographics->getLabelFontSizeSmall();
+		break;
+	case PartLabelFontSizeMedium:
+		fs = infographics->getLabelFontSizeMedium();
+		break;
+	case PartLabelFontSizeLarge:
+		fs = infographics->getLabelFontSizeLarge();
+		break;
+	default:
+		return;
 	}
 	m_font.setPointSize(fs);
 }
 
 void PartLabel::setFontPointSize(double pointSize) {
-    m_font.setPointSize(pointSize);
-    resetSvg();
+	m_font.setPointSize(pointSize);
+	resetSvg();
 }
 
 void PartLabel::setLabelDisplay(const QString & key) {
@@ -802,21 +823,21 @@ void PartLabel::setLabelDisplay(const QString & key) {
 QString mapToSVGWeight(int w) {
 	int percent = 50;
 	switch (w) {
-		case QFont::Light: 
-			percent = 25; 
-			break;
-		case QFont::Normal:	
-			return "normal";    // 50
-		case QFont::DemiBold: 
-			percent = 63;
-			break;
-		case QFont::Bold: 
-			return "bold";		// 75
-		case QFont::Black: 
-			percent = 87; 
-			break;
-		default:
-			return "normal";
+	case QFont::Light:
+		percent = 25;
+		break;
+	case QFont::Normal:
+		return "normal";    // 50
+	case QFont::DemiBold:
+		percent = 63;
+		break;
+	case QFont::Bold:
+		return "bold";		// 75
+	case QFont::Black:
+		percent = 87;
+		break;
+	default:
+		return "normal";
 	}
 
 	return QString::number((qRound(8 * percent / 100.0) * 100) + 100);   // 	normal | bold | bolder | lighter | 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900 | inherit
@@ -824,10 +845,14 @@ QString mapToSVGWeight(int w) {
 
 QString mapToSVGStyle(QFont::Style style) {
 	switch (style) {
-		case QFont::StyleNormal: return "normal";
-		case QFont::StyleOblique: return "oblique";
-		case QFont::StyleItalic: return "italic";
-		default: return "normal";
+	case QFont::StyleNormal:
+		return "normal";
+	case QFont::StyleOblique:
+		return "oblique";
+	case QFont::StyleItalic:
+		return "italic";
+	default:
+		return "normal";
 	}
 }
 
@@ -843,31 +868,46 @@ QString PartLabel::makeSvg(bool blackOnly, double dpi, double printerScale, bool
 	return svg;
 }
 
+bool PartLabel::migrateLabelOffset()
+{
+	// Fritzing before 1.0.0 moved labels by these arbitrary values,
+	// which resulted in text cut off at the top. With this migration
+	// we compensate for that when loading 0.9.10 projects or older.
+	// This way fonts are not cut anymore, but old projects still keep
+	// labels at the same position.
+	const double fontOffsetFactor=0.25;
+	qreal dy = m_font.pointSizeF()  * GraphicsUtils::SVGDPI / 72.0 * fontOffsetFactor;
+	QTransform t = sceneTransform();
+	QTransform labelTransform = QTransform(t.m11(), t.m12(), t.m13(), t.m21(), t.m22(), t.m23(), 0.0, 0.0, t.m33());
+	moveLabel(pos() + labelTransform.map(QPointF(0, -dy)), m_offset);
+	return true;
+}
 
-QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, double & w, double & h) 
+
+
+QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, double & w, double & h)
 {
 	if (m_displayText.isEmpty()) return "";
 
 	double pixels = m_font.pointSizeF() * printerScale / 72;
-	double y = pixels * 0.75;
-	//DebugDialog::debug(QString("initial y:%1").arg(y));
-	
-	QString svg = QString("<g font-size='%1' font-style='%2' font-weight='%3' fill='%4' font-family=\"'%5'\" id='%6' fill-opacity='1' stroke='none' >")
-		.arg(m_font.pointSizeF() * dpi / 72)
-		.arg(mapToSVGStyle(m_font.style()))
-		.arg(mapToSVGWeight(m_font.weight()))
-		.arg(blackOnly ? "#000000" : m_color.name())
-		.arg(InstalledFonts::InstalledFontsNameMapper.value(m_font.family()))
-		.arg(ViewLayer::viewLayerXmlNameFromID(m_viewLayerID)
+	double y = pixels;
+
+	QString svg =
+		QString("<g font-size='%1' font-style='%2' font-weight='%3' fill='%4' font-family=\"'%5'\" id='%6' fill-opacity='1' stroke='none' >")
+		.arg(QString::number(m_font.pointSizeF() * dpi / 72.0, 'f', 3),
+			mapToSVGStyle(m_font.style()),
+			mapToSVGWeight(m_font.weight()),
+			blackOnly ? "#000000" : m_color.name(),
+			InstalledFonts::InstalledFontsNameMapper.value(m_font.family()),
+			ViewLayer::viewLayerXmlNameFromID(m_viewLayerID)
 		);
 
 	w = 0;
 	QStringList texts = m_displayText.split("\n");
-	foreach (QString t, texts) {
+	for (const QString& t : texts) {
 		QString t1 = TextUtils::convertExtendedChars(TextUtils::escapeAnd(t));
 		svg += QString("<text x='0' y='%1'>%2</text>")
-			.arg(y * dpi / printerScale)
-			.arg(t1);
+				.arg(QString::number(y * dpi / printerScale, 'f', 3), t1);
 		y += pixels;
 		w = qMax(w, t.length() * pixels * 0.75);
 		//DebugDialog::debug(QString("\t%1, %2").arg(w).arg(y));
@@ -875,7 +915,10 @@ QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, d
 
 	svg += "</g>";
 
+	const double fontOffsetFactor=0.25;
 	h = y - (pixels / 2);
+	h -= fontOffsetFactor * pixels;
+
 
 	//QFontInfo fontInfo(m_font);
 	//DebugDialog::debug(QString("%1 match:%2 ps:%3 sty:%4 w:%5")
@@ -889,7 +932,7 @@ QString PartLabel::makeSvgAux(bool blackOnly, double dpi, double printerScale, d
 	return svg;
 
 	//DebugDialog::debug(QString("final:%1 %2 %3").arg(w).arg(h).arg(m_font.toString()));
-    
+
 }
 
 void PartLabel::resetSvg()
@@ -901,7 +944,7 @@ void PartLabel::resetSvg()
 
 	QString svg = TextUtils::makeSVGHeader(GraphicsUtils::SVGDPI, GraphicsUtils::StandardFritzingDPI, w, h) + innerSvg + "\n</svg>";
 
-	if (m_renderer == NULL) {
+	if (m_renderer == nullptr) {
 		m_renderer = new QSvgRenderer(this);
 	}
 
