@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2013 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,21 +16,13 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6956 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-04-07 12:14:50 +0200 (So, 07. Apr 2013) $
-
 ********************************************************************/
 
 #include "modelpart.h"
-#include "../debugdialog.h"
 #include "../connectors/connectorshared.h"
 #include "../connectors/busshared.h"
 #include "../connectors/bus.h"
 #include "../version/version.h"
-#include "../utils/folderutils.h"
 #include "../utils/textutils.h"
 #include "../items/itembase.h"
 #include "../items/partfactory.h"
@@ -43,10 +35,10 @@ const int ModelPart::indexMultiplier = 10;
 QStringList ModelPart::m_possibleFolders;
 
 typedef QHash<QString, ModelPartList*> InstanceTitleIncrementHash;
-static QHash<QObject *, InstanceTitleIncrementHash *> AllInstanceTitleIncrements; 
+static QHash<QObject *, InstanceTitleIncrementHash *> AllInstanceTitleIncrements;
 InstanceTitleIncrementHash NullInstanceTitleIncrements;
 
-static const QRegExp InstanceTitleRegExp("^(.*[^\\d])(\\d+)$");
+static const QRegularExpression InstanceTitleRegExp("^(.*[^\\d])(\\d+)$");
 
 static const QList<ViewImage *> EmptyViewImages;
 
@@ -56,7 +48,7 @@ ModelPart::ModelPart(ItemType type)
 	: QObject()
 {
 	commonInit(type);
-	m_modelPartShared = NULL;
+	m_modelPartShared = nullptr;
 	m_index = m_nextIndex++;
 }
 
@@ -65,12 +57,12 @@ ModelPart::ModelPart(QDomDocument & domDocument, const QString & path, ItemType
 {
 	commonInit(type);
 	m_modelPartShared = new ModelPartShared(domDocument, path);
-    m_modelPartShared->addOwner(this);
+	m_modelPartShared->addOwner(this);
 }
 
 void ModelPart::commonInit(ItemType type) {
 	m_type = type;
-	m_locationFlags = 0;
+	m_locationFlags = QFlags<LocationFlag>();
 	m_indexSynched = false;
 }
 
@@ -82,13 +74,13 @@ ModelPart::~ModelPart() {
 	InstanceTitleIncrementHash * itih = AllInstanceTitleIncrements.value(this);
 	if (itih) {
 		AllInstanceTitleIncrements.remove(this);
-		foreach (ModelPartList * list, itih->values()) {
+		Q_FOREACH (ModelPartList * list, itih->values()) {
 			delete list;
 		}
 		delete itih;
 	}
 
-	foreach (Connector * connector, m_connectorHash.values()) {
+	Q_FOREACH (Connector * connector, m_connectorHash.values()) {
 		delete connector;
 	}
 	m_connectorHash.clear();
@@ -97,37 +89,37 @@ ModelPart::~ModelPart() {
 }
 
 const QString & ModelPart::moduleID() const {
-	if (m_modelPartShared != NULL) return m_modelPartShared->moduleID();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->moduleID();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::label() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->label();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->label();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::author() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->author();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->author();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::taxonomy() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->taxonomy();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->taxonomy();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::uri() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->uri();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->uri();
 
 	return ___emptyString___;
 }
 
 const QDate & ModelPart::date() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->date();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->date();
 
 	static QDate tempDate;
 	tempDate = QDate::currentDate();
@@ -139,11 +131,11 @@ void ModelPart::setItemType(ItemType t) {
 }
 
 void ModelPart::copy(ModelPart * modelPart) {
-	if (modelPart == NULL) return;
+	if (modelPart == nullptr) return;
 
 	m_type = modelPart->itemType();
 	m_modelPartShared = modelPart->modelPartShared();
-    if (m_modelPartShared) m_modelPartShared->addOwner(this);
+	if (m_modelPartShared) m_modelPartShared->addOwner(this);
 	m_locationFlags = modelPart->m_locationFlags;
 }
 
@@ -158,7 +150,7 @@ void ModelPart::copyStuff(ModelPart * modelPart) {
 ModelPartShared * ModelPart::modelPartShared() {
 	if(!m_modelPartShared) {
 		m_modelPartShared = new ModelPartShared();
-        m_modelPartShared->addOwner(this);
+		m_modelPartShared->addOwner(this);
 	}
 	return m_modelPartShared;
 }
@@ -168,8 +160,8 @@ ModelPartSharedRoot * ModelPart::modelPartSharedRoot() {
 }
 
 void ModelPart::setModelPartShared(ModelPartShared * modelPartShared) {
-	m_modelPartShared = modelPartShared;    
-    if (modelPartShared) m_modelPartShared->addOwner(this);
+	m_modelPartShared = modelPartShared;
+	if (modelPartShared) m_modelPartShared->addOwner(this);
 }
 
 void ModelPart::addViewItem(ItemBase * item) {
@@ -181,25 +173,25 @@ void ModelPart::removeViewItem(ItemBase * item) {
 }
 
 ItemBase * ModelPart::viewItem(QGraphicsScene * scene) {
-	foreach (ItemBase * itemBase, m_viewItems) {
+	Q_FOREACH (ItemBase * itemBase, m_viewItems) {
 		if (itemBase->scene() == scene) return itemBase;
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 ItemBase * ModelPart::viewItem(ViewLayer::ViewID viewID) {
-	foreach (ItemBase * itemBase, m_viewItems) {
+	Q_FOREACH (ItemBase * itemBase, m_viewItems) {
 		if (itemBase->viewID() == viewID) return itemBase;
 	}
 
-	return NULL;
+	return nullptr;
 }
 
-void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & streamWriter, bool startDocument) {
+void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & streamWriter, bool startDocument, bool flipAware) {
 	if (startDocument) {
 		streamWriter.writeStartDocument();
-    	streamWriter.writeStartElement("module");
+		streamWriter.writeStartElement("module");
 		streamWriter.writeAttribute("fritzingVersion", Version::versionString());
 		ModelPartSharedRoot * root = modelPartSharedRoot();
 		if (root) {
@@ -214,14 +206,14 @@ void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & strea
 		if(!title.isNull() && !title.isEmpty()) {
 			streamWriter.writeTextElement("title",title);
 		}
-		
-		emit startSaveInstances(fileName, this, streamWriter);
+
+		Q_EMIT startSaveInstances(fileName, this, streamWriter);
 
 		streamWriter.writeStartElement("instances");
 	}
 
-	if (parent() != NULL) {  // m_viewItems.size() > 0
-		saveInstance(streamWriter);
+	if (parent() != nullptr) {  // m_viewItems.size() > 0
+		saveInstance(streamWriter, flipAware);
 	}
 
 	QList<QObject *> children = this->children();
@@ -231,10 +223,10 @@ void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & strea
 
 	QList<QObject *>::const_iterator i;
 	for (i = children.constBegin(); i != children.constEnd(); ++i) {
-		ModelPart* mp = qobject_cast<ModelPart *>(*i);
-		if (mp == NULL) continue;
+		auto* mp = qobject_cast<ModelPart *>(*i);
+		if (mp == nullptr) continue;
 
-		mp->saveInstances(fileName, streamWriter, false);
+		mp->saveInstances(fileName, streamWriter, false, flipAware);
 	}
 
 
@@ -245,26 +237,26 @@ void ModelPart::saveInstances(const QString & fileName, QXmlStreamWriter & strea
 	}
 }
 
-void ModelPart::saveInstance(QXmlStreamWriter & streamWriter) 
+void ModelPart::saveInstance(QXmlStreamWriter & streamWriter, bool flipAware)
 {
 	if (localProp("ratsnest").toBool()) {
 		return;				// don't save virtual wires
 	}
 
 	streamWriter.writeStartElement("instance");
-	if (m_modelPartShared != NULL) {
+	if (m_modelPartShared != nullptr) {
 		QString moduleIdRef = m_modelPartShared->moduleID();
-        moduleIdRef.remove(PartFactory::OldSchematicPrefix);
+		moduleIdRef.remove(PartFactory::OldSchematicPrefix);
 		streamWriter.writeAttribute("moduleIdRef", moduleIdRef);
 		streamWriter.writeAttribute("modelIndex", QString::number(m_index));
 		streamWriter.writeAttribute("path", m_modelPartShared->path());
-        if (m_modelPartShared->flippedSMD()) {
+		if (m_modelPartShared->flippedSMD()) {
 			streamWriter.writeAttribute("flippedSMD", "true");
 		}
 	}
 
 	bool writeLocal = false;
-	foreach (Connector * connector, this->connectors()) {
+	Q_FOREACH (Connector * connector, this->connectors()) {
 		if (!connector->connectorLocalName().isEmpty()) {
 			writeLocal = true;
 			break;
@@ -273,18 +265,18 @@ void ModelPart::saveInstance(QXmlStreamWriter & streamWriter)
 
 	if (writeLocal) {
 		streamWriter.writeStartElement("localConnectors");
-		foreach (Connector * connector, this->connectors()) {
+		Q_FOREACH (Connector * connector, this->connectors()) {
 			if (!connector->connectorLocalName().isEmpty()) {
 				streamWriter.writeStartElement("localConnector");
 				streamWriter.writeAttribute("id", connector->connectorSharedID());
-				streamWriter.writeAttribute("name", TextUtils::stripNonValidXMLCharacters(TextUtils::escapeAnd(connector->connectorLocalName())));
+				streamWriter.writeAttribute("name", TextUtils::stripNonValidXMLCharacters(connector->connectorLocalName()));
 				streamWriter.writeEndElement();
 			}
 		}
 		streamWriter.writeEndElement();
 	}
 
-	foreach (QByteArray byteArray, dynamicPropertyNames()) {
+	Q_FOREACH (QByteArray byteArray, dynamicPropertyNames()) {
 		streamWriter.writeStartElement("property");
 		streamWriter.writeAttribute("name",  byteArray.data());
 		streamWriter.writeAttribute("value", property(byteArray.data()).toString());
@@ -305,8 +297,8 @@ void ModelPart::saveInstance(QXmlStreamWriter & streamWriter)
 
 	// tell the views to write themselves out
 	streamWriter.writeStartElement("views");
-	foreach (ItemBase * itemBase, m_viewItems) {
-		itemBase->saveInstance(streamWriter);
+	Q_FOREACH (ItemBase * itemBase, m_viewItems) {
+		itemBase->saveInstance(streamWriter, flipAware);
 	}
 	streamWriter.writeEndElement();		// views
 	streamWriter.writeEndElement();		//instance
@@ -343,51 +335,51 @@ void ModelPart::writeNestedTag(QXmlStreamWriter & streamWriter, QString tagName,
 void ModelPart::saveAsPart(QXmlStreamWriter & streamWriter, bool startDocument) {
 	if (startDocument) {
 		streamWriter.writeStartDocument();
-    	streamWriter.writeStartElement("module");
+		streamWriter.writeStartElement("module");
 		streamWriter.writeAttribute("fritzingVersion", Version::versionString());
-        QString moduleID = m_modelPartShared->moduleID();
-        moduleID.remove(PartFactory::OldSchematicPrefix);
+		QString moduleID = m_modelPartShared->moduleID();
+		moduleID.remove(PartFactory::OldSchematicPrefix);
 		streamWriter.writeAttribute("moduleId", moduleID);
-    	writeTag(streamWriter,"version",m_modelPartShared->version());
-    	writeTag(streamWriter,"author",m_modelPartShared->author());
-    	writeTag(streamWriter,"title",title());
-    	writeTag(streamWriter,"label",m_modelPartShared->label());
-    	writeTag(streamWriter,"date",m_modelPartShared->dateAsStr());
+		writeTag(streamWriter,"version",m_modelPartShared->version());
+		writeTag(streamWriter,"author",m_modelPartShared->author());
+		writeTag(streamWriter,"title",title());
+		writeTag(streamWriter,"label",m_modelPartShared->label());
+		writeTag(streamWriter,"date",m_modelPartShared->dateAsStr());
 
-    	writeNestedTag(streamWriter,"tags",m_modelPartShared->tags(),"tag");
-    	writeNestedTag(streamWriter,"properties",m_modelPartShared->properties(),"property","name");
+		writeNestedTag(streamWriter,"tags",m_modelPartShared->tags(),"tag");
+		writeNestedTag(streamWriter,"properties",m_modelPartShared->properties(),"property","name");
 
-    	writeTag(streamWriter,"taxonomy",m_modelPartShared->taxonomy());
-    	writeTag(streamWriter,"description",m_modelPartShared->description());
+		writeTag(streamWriter,"taxonomy",m_modelPartShared->taxonomy());
+		writeTag(streamWriter,"description",m_modelPartShared->description());
 
-        QString spice = m_modelPartShared->spice();
-        if (!spice.isEmpty()) {
-            streamWriter.writeStartElement("spice");
-            QStringList lines = spice.split("\r",QString::SkipEmptyParts);
-            foreach (QString line, lines) {
-                writeTag(streamWriter, "line", line);
-            }
-            QString spiceModel = m_modelPartShared->spiceModel();
-            if (!spiceModel.isEmpty()) {
-                lines = spiceModel.split("\r",QString::SkipEmptyParts);
-                foreach (QString line, lines) {
-                    writeTag(streamWriter, "model", line);
-                }
-            }
-            streamWriter.writeEndElement();
-        }
+		QString spice = m_modelPartShared->spice();
+		if (!spice.isEmpty()) {
+			streamWriter.writeStartElement("spice");
+			QStringList lines = spice.split("\r",Qt::SkipEmptyParts);
+			Q_FOREACH (QString line, lines) {
+				writeTag(streamWriter, "line", line);
+			}
+			QString spiceModel = m_modelPartShared->spiceModel();
+			if (!spiceModel.isEmpty()) {
+				lines = spiceModel.split("\r",Qt::SkipEmptyParts);
+				Q_FOREACH (QString line, lines) {
+					writeTag(streamWriter, "model", line);
+				}
+			}
+			streamWriter.writeEndElement();
+		}
 
 
-    	writeTag(streamWriter,"spice",m_modelPartShared->spice());
-    	writeTag(streamWriter,"url",m_modelPartShared->url());
+		writeTag(streamWriter,"spice",m_modelPartShared->spice());
+		writeTag(streamWriter,"url",m_modelPartShared->url());
 	}
 
 	if (m_viewItems.size() > 0) {
 		if (startDocument) {
 			streamWriter.writeStartElement("views");
 		}
-		for (int i = 0; i < m_viewItems.size(); i++) {
-			ItemBase * item = m_viewItems[i];
+		for (auto & m_viewItem : m_viewItems) {
+			ItemBase * item = m_viewItem;
 			item->writeXml(streamWriter);
 		}
 
@@ -398,7 +390,7 @@ void ModelPart::saveAsPart(QXmlStreamWriter & streamWriter, bool startDocument)
 		streamWriter.writeStartElement("connectors");
 		const QList< QPointer<ConnectorShared> > connectors = m_modelPartShared->connectorsShared();
 		for (int i = 0; i < connectors.count(); i++) {
-			Connector * connector = new Connector(connectors[i], this);
+			auto * connector = new Connector(connectors[i], this);
 			connector->saveAsPart(streamWriter);
 			delete connector;
 		}
@@ -406,9 +398,9 @@ void ModelPart::saveAsPart(QXmlStreamWriter & streamWriter, bool startDocument)
 	}
 
 	QList<QObject *>::const_iterator i;
-    for (i = children().constBegin(); i != children().constEnd(); ++i) {
-		ModelPart * mp = qobject_cast<ModelPart *>(*i);
-		if (mp == NULL) continue;
+	for (i = children().constBegin(); i != children().constEnd(); ++i) {
+		auto * mp = qobject_cast<ModelPart *>(*i);
+		if (mp == nullptr) continue;
 
 		mp->saveAsPart(streamWriter, false);
 	}
@@ -421,40 +413,40 @@ void ModelPart::saveAsPart(QXmlStreamWriter & streamWriter, bool startDocument)
 }
 
 void ModelPart::initConnectors(bool force) {
-	if(m_modelPartShared == NULL) return;
+	if(m_modelPartShared == nullptr) return;
 
 	if(force) {
-		foreach (Connector * connector, m_connectorHash.values()) {
+		Q_FOREACH (Connector * connector, m_connectorHash.values()) {
 			// due to craziness in the parts editor
 			// m_deletedConnectors.append(connector);
 			delete connector;
 		}
-		m_connectorHash.clear();		
+		m_connectorHash.clear();
 		clearBuses();
 	}
 	if(m_connectorHash.count() > 0) return;		// already done
 
 	m_modelPartShared->initConnectors();
-	foreach (ConnectorShared * connectorShared, m_modelPartShared->connectorsShared()) {
-		Connector * connector = new Connector(connectorShared, this);
+	Q_FOREACH (ConnectorShared * connectorShared, m_modelPartShared->connectorsShared()) {
+		auto * connector = new Connector(connectorShared, this);
 		m_connectorHash.insert(connectorShared->id(), connector);
 	}
 	initBuses();
 }
 
 void ModelPart::clearBuses() {
-	foreach (Bus * bus, m_busHash.values()) {
+	Q_FOREACH (Bus * bus, m_busHash.values()) {
 		delete bus;
 	}
 	m_busHash.clear();
 }
 
 void ModelPart::initBuses() {
-	foreach (Connector * connector, m_connectorHash.values()) {
+	Q_FOREACH (Connector * connector, m_connectorHash.values()) {
 		BusShared * busShared = connector->connectorShared()->bus();
-		if (busShared != NULL) {
+		if (busShared != nullptr) {
 			Bus * bus = m_busHash.value(busShared->id());
-			if (bus == NULL) {
+			if (bus == nullptr) {
 				bus = new Bus(busShared, this);
 				m_busHash.insert(busShared->id(), bus);
 			}
@@ -515,7 +507,7 @@ const QDomElement & ModelPart::instanceDomElement() {
 
 const QString & ModelPart::fritzingVersion() {
 
-	if (m_modelPartShared != NULL) return m_modelPartShared->fritzingVersion();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->fritzingVersion();
 
 	return ___emptyString___;
 }
@@ -523,7 +515,7 @@ const QString & ModelPart::fritzingVersion() {
 const QString & ModelPart::title() {
 	if (!m_localTitle.isEmpty()) return m_localTitle;
 
-	if (m_modelPartShared != NULL) return m_modelPartShared->title();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->title();
 
 	return m_localTitle;
 }
@@ -533,49 +525,49 @@ void ModelPart::setLocalTitle(const QString & localTitle) {
 }
 
 const QString & ModelPart::version() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->version();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->version();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::path() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->path();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->path();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::description() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->description();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->description();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::spice() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->spice();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->spice();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::spiceModel() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->spiceModel();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->spiceModel();
 
 	return ___emptyString___;
 }
 
 const QString & ModelPart::url() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->url();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->url();
 
 	return ___emptyString___;
 }
 
 const QStringList & ModelPart::tags() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->tags();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->tags();
 
 	return ___emptyStringList___;
 }
 
 const QHash<QString,QString> & ModelPart::properties() const {
-	if (m_modelPartShared != NULL) return m_modelPartShared->properties();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->properties();
 
 	return ___emptyStringHash___;
 }
@@ -593,7 +585,7 @@ Bus * ModelPart::bus(const QString & busID) {
 }
 
 bool ModelPart::ignoreTerminalPoints() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->ignoreTerminalPoints();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->ignoreTerminalPoints();
 
 	return true;
 }
@@ -647,56 +639,56 @@ void ModelPart::setLocationFlag(bool setting, LocationFlag flag) {
 }
 
 bool ModelPart::hasViewID(ViewLayer::ViewID viewID) {
-    if (m_modelPartShared) return m_modelPartShared->hasViewID(viewID);
+	if (m_modelPartShared) return m_modelPartShared->hasViewID(viewID);
 
-    return false;
+	return false;
 
 }
 
 bool ModelPart::canFlipVertical(ViewLayer::ViewID viewID) {
-    if (m_modelPartShared) return m_modelPartShared->canFlipVertical(viewID);
+	if (m_modelPartShared) return m_modelPartShared->canFlipVertical(viewID);
 
-    return false;
+	return false;
 
 }
 
 bool ModelPart::canFlipHorizontal(ViewLayer::ViewID viewID) {
-    if (m_modelPartShared) return m_modelPartShared->canFlipHorizontal(viewID);
+	if (m_modelPartShared) return m_modelPartShared->canFlipHorizontal(viewID);
 
-    return false;
+	return false;
 }
 
 LayerList ModelPart::viewLayers(ViewLayer::ViewID viewID) {
-    if (m_modelPartShared) return m_modelPartShared->viewLayers(viewID);
+	if (m_modelPartShared) return m_modelPartShared->viewLayers(viewID);
 
-    LayerList layerList;
-    return layerList;
+	LayerList layerList;
+	return layerList;
 }
 
 QString ModelPart::imageFileName(ViewLayer::ViewID viewID) {
-    if (m_modelPartShared) return m_modelPartShared->imageFileName(viewID);
+	if (m_modelPartShared) return m_modelPartShared->imageFileName(viewID);
 
-    return "";
+	return "";
 }
 
 QString ModelPart::imageFileName(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID) {
-    if (m_modelPartShared) return m_modelPartShared->imageFileName(viewID, viewLayerID);
+	if (m_modelPartShared) return m_modelPartShared->imageFileName(viewID, viewLayerID);
 
-    return "";
+	return "";
 }
 
 bool ModelPart::anySticky(ViewLayer::ViewID viewID) {
-    if (m_modelPartShared) return m_modelPartShared->anySticky(viewID);
+	if (m_modelPartShared) return m_modelPartShared->anySticky(viewID);
 
-    return false;
+	return false;
 }
 
 QList<ModelPart*> ModelPart::getAllParts() {
 	QList<ModelPart*> retval;
 	QList<QObject *>::const_iterator i;
 	for (i = children().constBegin(); i != children().constEnd(); ++i) {
-		ModelPart* mp = qobject_cast<ModelPart *>(*i);
-		if (mp == NULL) continue;
+		auto* mp = qobject_cast<ModelPart *>(*i);
+		if (mp == nullptr) continue;
 		retval << mp;
 	}
 
@@ -707,8 +699,8 @@ QList<ModelPart*> ModelPart::getAllNonCoreParts() {
 	QList<ModelPart*> retval;
 	QList<QObject *>::const_iterator i;
 	for (i = children().constBegin(); i != children().constEnd(); ++i) {
-		ModelPart* mp = qobject_cast<ModelPart *>(*i);
-		if (mp == NULL) continue;
+		auto* mp = qobject_cast<ModelPart *>(*i);
+		if (mp == nullptr) continue;
 
 		if(!mp->isCore()) {
 			retval << mp;
@@ -719,7 +711,7 @@ QList<ModelPart*> ModelPart::getAllNonCoreParts() {
 }
 
 bool ModelPart::hasViewID(long id) {
-	foreach (ItemBase * item, m_viewItems) {
+	Q_FOREACH (ItemBase * item, m_viewItems) {
 		if (item->id() == id) return true;
 	}
 
@@ -738,47 +730,47 @@ void ModelPart::setInstanceText(QString text) {
 	m_instanceText = text;
 }
 
-void ModelPart::clearOldInstanceTitle(const QString & title) 
+void ModelPart::clearOldInstanceTitle(const QString & title)
 {
-	InstanceTitleIncrementHash * itih = NULL;
-	if (parent() == NULL) {
+	InstanceTitleIncrementHash * itih = nullptr;
+	if (parent() == nullptr) {
 		itih = &NullInstanceTitleIncrements;
 	}
 	else {
-		itih = AllInstanceTitleIncrements.value(parent(), NULL);
+		itih = AllInstanceTitleIncrements.value(parent(), nullptr);
 	}
 
-	if (itih == NULL) return;
+	if (itih == nullptr) return;
 
 	//DebugDialog::debug(QString("clearing title:%1 ix:%2").arg(title).arg(modelPart->modelIndex()));
 	QString prefix = title;
-	int ix = InstanceTitleRegExp.indexIn(title);
-	if (ix >= 0) {
-		prefix = InstanceTitleRegExp.cap(1);
+	QRegularExpressionMatch match;
+	if (title.contains(InstanceTitleRegExp, &match)) {
+		prefix = match.captured(1);
 	}
-	ModelPartList * modelParts = itih->value(prefix, NULL);
+	ModelPartList * modelParts = itih->value(prefix, nullptr);
 	if (modelParts) {
 		modelParts->removeOne(this);
 		//DebugDialog::debug(QString("\tc:%1").arg(modelParts->count()));
 	}
 }
 
-ModelPartList * ModelPart::ensureInstanceTitleIncrements(const QString & prefix) 
+ModelPartList * ModelPart::ensureInstanceTitleIncrements(const QString & prefix)
 {
-	InstanceTitleIncrementHash * itih = NULL;
-	if (parent() == NULL) {
+	InstanceTitleIncrementHash * itih = nullptr;
+	if (parent() == nullptr) {
 		itih = &NullInstanceTitleIncrements;
 	}
 	else {
-		itih = AllInstanceTitleIncrements.value(parent(), NULL);
-		if (itih == NULL) {
+		itih = AllInstanceTitleIncrements.value(parent(), nullptr);
+		if (itih == nullptr) {
 			itih = new InstanceTitleIncrementHash;
 			AllInstanceTitleIncrements.insert(parent(), itih);
 		}
 	}
 
-	ModelPartList * modelParts = itih->value(prefix, NULL);
-	if (modelParts == NULL) {
+	ModelPartList * modelParts = itih->value(prefix, nullptr);
+	if (modelParts == nullptr) {
 		modelParts =  new ModelPartList;
 		itih->insert(prefix, modelParts);
 	}
@@ -786,67 +778,69 @@ ModelPartList * ModelPart::ensureInstanceTitleIncrements(const QString & prefix)
 }
 
 void ModelPart::setInstanceTitle(QString title, bool initial) {
-    if (initial) {
-        if (setSubpartInstanceTitle()) return;
-    }
+	if (initial) {
+		if (setSubpartInstanceTitle()) return;
+	}
 
-    if (title.compare(m_instanceTitle) == 0) return;
+	if (title.compare(m_instanceTitle) == 0) return;
 
 	clearOldInstanceTitle(m_instanceTitle);
 
 	m_instanceTitle = title;
 
 	QString prefix = title;
-	int ix = InstanceTitleRegExp.indexIn(title);
-	if (ix >= 0) {
-		prefix = InstanceTitleRegExp.cap(1);
+
+	QRegularExpressionMatch match;
+	if (title.contains(InstanceTitleRegExp, &match)) {
+		prefix = match.captured(1);
 		ModelPartList * modelParts = ensureInstanceTitleIncrements(prefix);
 		modelParts->append(this);
 	}
 	//DebugDialog::debug(QString("adding title:%1 ix:%2 c:%3").arg(title).arg(modelIndex()).arg(modelParts->count()));
 
-    if (m_modelPartShared && m_modelPartShared->hasSubparts()) {
-        if (m_viewItems.count() > 0) {
-            ItemBase * itemBase = m_viewItems.last();
-            if (itemBase) {
-                foreach (ItemBase * subpart, itemBase->subparts()) {
-                    subpart->setInstanceTitle("", true);   // will end up calling setSubpartInstanceTitle()
-                }
-            }
-        }
-    }
+	if (m_modelPartShared && m_modelPartShared->hasSubparts()) {
+		if (m_viewItems.count() > 0) {
+			ItemBase * itemBase = m_viewItems.last();
+			if (itemBase) {
+				Q_FOREACH (ItemBase * subpart, itemBase->subparts()) {
+					subpart->setInstanceTitle("", true);   // will end up calling setSubpartInstanceTitle()
+				}
+			}
+		}
+	}
 }
 
 bool ModelPart::setSubpartInstanceTitle() {
-    if (m_modelPartShared == NULL) return false;
-    if (m_modelPartShared->superpart() == NULL) return false;
-    if (m_viewItems.count() <= 0) return false;
-                
-    ItemBase * itemBase = m_viewItems.last();
-    if (itemBase == NULL) return false;
+	if (m_modelPartShared == nullptr) return false;
+	if (m_modelPartShared->superpart() == nullptr) return false;
+	if (m_viewItems.count() <= 0) return false;
 
-    itemBase = itemBase->superpart();
-    if (itemBase == NULL) return false;
+	ItemBase * itemBase = m_viewItems.last();
+	if (itemBase == nullptr) return false;
 
-    QString superTitle = itemBase->instanceTitle();
-    if (superTitle.isEmpty()) return false;
+	itemBase = itemBase->superpart();
+	if (itemBase == nullptr) return false;
 
-    QString label = m_modelPartShared->label();
-    if (label.isEmpty()) return false;
+	QString superTitle = itemBase->instanceTitle();
+	if (superTitle.isEmpty()) return false;
 
-    m_instanceTitle = superTitle + "_" + label;
-    return true;
+	QString label = m_modelPartShared->label();
+	if (label.isEmpty()) return false;
+
+	m_instanceTitle = superTitle + "_" + label;
+	return true;
 }
 
 QString ModelPart::getNextTitle(const QString & title) {
 	QString prefix = title;
-	int ix = InstanceTitleRegExp.indexIn(title);
-	if (ix >= 0) {
-		prefix = InstanceTitleRegExp.cap(1);
+
+	QRegularExpressionMatch match;
+	if (title.contains(InstanceTitleRegExp, &match)) {
+		prefix = match.captured(1);
 	}
 	else {
 		bool allDigits = true;
-		foreach (QChar c, title) {
+		Q_FOREACH (QChar c, title) {
 			if (!c.isDigit()) {
 				allDigits = false;
 				break;
@@ -857,12 +851,12 @@ QString ModelPart::getNextTitle(const QString & title) {
 		}
 	}
 
-	// TODO: if this were a sorted list, 
+	// TODO: if this were a sorted list,
 	ModelPartList * modelParts = ensureInstanceTitleIncrements(prefix);
 	int highestSoFar = 0;
 	bool gotNull = false;
-	foreach (ModelPart * modelPart, *modelParts) {
-		if (modelPart == NULL) {
+	Q_FOREACH (ModelPart * modelPart, *modelParts) {
+		if (modelPart == nullptr) {
 			gotNull = true;
 			continue;
 		}
@@ -876,7 +870,7 @@ QString ModelPart::getNextTitle(const QString & title) {
 	}
 
 	if (gotNull) {
-		modelParts->removeAll(NULL);
+		modelParts->removeAll(nullptr);
 	}
 
 	//DebugDialog::debug(QString("returning increment %1, %2").arg(prefix).arg(highestSoFar + 1));
@@ -888,7 +882,7 @@ void ModelPart::setOrderedChildren(QList<QObject*> children) {
 }
 
 void ModelPart::setLocalProp(const char * name, const QVariant & value) {
-    //DebugDialog::debug(QString("mp set prop %1 %2").arg(name).arg(value.toString()));
+	//DebugDialog::debug(QString("mp set prop %1 %2").arg(name).arg(value.toString()));
 	QObject::setProperty(name, value);
 }
 
@@ -915,19 +909,19 @@ const QStringList & ModelPart::possibleFolders() {
 }
 
 const QString & ModelPart::replacedby() {
-	if (m_modelPartShared != NULL) return m_modelPartShared->replacedby();
+	if (m_modelPartShared != nullptr) return m_modelPartShared->replacedby();
 
 	return ___emptyString___;
 }
 
 bool ModelPart::isObsolete() {
-	if (m_modelPartShared != NULL) return !m_modelPartShared->replacedby().isEmpty();
+	if (m_modelPartShared != nullptr) return !m_modelPartShared->replacedby().isEmpty();
 
 	return false;
 }
 
 bool ModelPart::flippedSMD() {
-	if (m_modelPartShared != NULL) {
+	if (m_modelPartShared != nullptr) {
 		return m_modelPartShared->flippedSMD();
 	}
 
@@ -935,7 +929,7 @@ bool ModelPart::flippedSMD() {
 }
 
 bool ModelPart::needsCopper1() {
-	if (m_modelPartShared != NULL) {
+	if (m_modelPartShared != nullptr) {
 		return m_modelPartShared->needsCopper1();
 	}
 
@@ -943,38 +937,38 @@ bool ModelPart::needsCopper1() {
 }
 
 bool ModelPart::hasViewFor(ViewLayer::ViewID viewID) {
-	if (m_modelPartShared == NULL) return false;
+	if (m_modelPartShared == nullptr) return false;
 
 	return m_modelPartShared->hasViewFor(viewID);
 }
 
 bool ModelPart::hasViewFor(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID) {
-	if (m_modelPartShared == NULL) return false;
+	if (m_modelPartShared == nullptr) return false;
 
 	return m_modelPartShared->hasViewFor(viewID, viewLayerID);
 }
 
 QString ModelPart::hasBaseNameFor(ViewLayer::ViewID viewID) {
-	if (m_modelPartShared == NULL) return ___emptyString___;
+	if (m_modelPartShared == nullptr) return ___emptyString___;
 
 	return m_modelPartShared->hasBaseNameFor(viewID);
 }
 
 const QStringList & ModelPart::displayKeys() {
-	if (m_modelPartShared == NULL) return ___emptyStringList___;
+	if (m_modelPartShared == nullptr) return ___emptyStringList___;
 
 	return m_modelPartShared->displayKeys();
 }
 
-ModelPart::ItemType ModelPart::itemType() const 
-{ 
-	return m_type; 
+ModelPart::ItemType ModelPart::itemType() const
+{
+	return m_type;
 };
 
 void ModelPart::setConnectorLocalName(const QString & id, const QString & name)
 {
 	if (id.isEmpty()) return;
-	Connector * connector = m_connectorHash.value(id, NULL);
+	Connector * connector = m_connectorHash.value(id, nullptr);
 	if (connector) {
 		connector->setConnectorLocalName(name);
 	}
@@ -984,7 +978,7 @@ QString ModelPart::connectorLocalName(const QString & id)
 {
 	if (id.isEmpty()) return "";
 
-	Connector * connector = m_connectorHash.value(id, NULL);
+	Connector * connector = m_connectorHash.value(id, nullptr);
 	if (connector) {
 		return connector->connectorLocalName();
 	}
@@ -992,7 +986,7 @@ QString ModelPart::connectorLocalName(const QString & id)
 	return "";
 }
 
-QString ModelPart::family(){
+QString ModelPart::family() {
 	if (m_modelPartShared) return m_modelPartShared->family();
 
 	return "";
@@ -1003,79 +997,69 @@ bool ModelPart::hasViewItems() {
 }
 
 void ModelPart::setDBID(qulonglong dbid) {
-    if (m_modelPartShared) m_modelPartShared->setDBID(dbid);
+	if (m_modelPartShared) m_modelPartShared->setDBID(dbid);
 }
 
 qulonglong ModelPart::dbid() {
-    if (m_modelPartShared) return m_modelPartShared->dbid();
+	if (m_modelPartShared) return m_modelPartShared->dbid();
 
-    return 0;
+	return 0;
 }
 
 const QList<ViewImage *> ModelPart::viewImages() {
-    if (m_modelPartShared) return m_modelPartShared->viewImages();
+	if (m_modelPartShared) return m_modelPartShared->viewImages();
 
-    return EmptyViewImages;
+	return EmptyViewImages;
 }
 
 void ModelPart::setViewImage(ViewImage * viewImage) {
-    if (m_modelPartShared) m_modelPartShared->setViewImage(viewImage);
+	if (m_modelPartShared) m_modelPartShared->setViewImage(viewImage);
 }
 
 void ModelPart::setTag(const QString & tag)
 {
-    if (m_modelPartShared) m_modelPartShared->setTag(tag);
+	if (m_modelPartShared) m_modelPartShared->setTag(tag);
 }
 
 void ModelPart::setProperty(const QString & name, const QString &value, bool showInLabel)
 {
-    if (m_modelPartShared) m_modelPartShared->setProperty(name, value, showInLabel);
+	if (m_modelPartShared) m_modelPartShared->setProperty(name, value, showInLabel);
 }
 
 bool ModelPart::showInLabel(const QString & propertyName) {
-    if (m_modelPartShared) return m_modelPartShared->showInLabel(propertyName);
+	if (m_modelPartShared) return m_modelPartShared->showInLabel(propertyName);
 
-    return false;
+	return false;
 }
 
 void ModelPart::addConnector(Connector * connector) {
-    m_connectorHash.insert(connector->connectorSharedID(), connector);
+	m_connectorHash.insert(connector->connectorSharedID(), connector);
 
-    if (m_modelPartShared) m_modelPartShared->addConnector(connector->connectorShared());
+	if (m_modelPartShared) m_modelPartShared->addConnector(connector->connectorShared());
 }
 
 void ModelPart::flipSMDAnd() {
-    if (m_modelPartShared) m_modelPartShared->flipSMDAnd();
+	if (m_modelPartShared) m_modelPartShared->flipSMDAnd();
 }
 
 void ModelPart::setImageFileName(ViewLayer::ViewID viewID, const QString & filename) {
-    if (m_modelPartShared) m_modelPartShared->setImageFileName(viewID, filename);
-}
-
-void ModelPart::lookForZeroConnector() {
-    if (m_modelPartShared) m_modelPartShared->lookForZeroConnector();
-}
-
-bool ModelPart::hasZeroConnector() {
-    if (m_modelPartShared) return m_modelPartShared->hasZeroConnector();
-
-    return false;
+	if (m_modelPartShared) m_modelPartShared->setImageFileName(viewID, filename);
 }
 
 void ModelPart::killViewItems() {
-	foreach (ItemBase * itemBase, m_viewItems) {
+	Q_FOREACH (ItemBase * itemBase, m_viewItems) {
 		if (itemBase) delete itemBase;
 	}
 
-    m_viewItems.clear();
+	m_viewItems.clear();
 }
 
 bool ModelPart::hasSubparts() {
-    if (m_modelPartShared) return m_modelPartShared->hasSubparts();
+	if (m_modelPartShared) return m_modelPartShared->hasSubparts();
 
-    return false;
+	return false;
 }
 
 void ModelPart::setSubpartID(const QString & id) {
-    if (m_modelPartShared) m_modelPartShared->setSubpartID(id);
+	if (m_modelPartShared) m_modelPartShared->setSubpartID(id);
 }
