@@ -1,7 +1,7 @@
 /*******************************************************************
 
 Part of the Fritzing project - http://fritzing.org
-Copyright (c) 2007-2012 Fachhochschule Potsdam - http://fh-potsdam.de
+Copyright (c) 2007-2019 Fritzing
 
 Fritzing is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,12 +16,6 @@ GNU General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Fritzing.  If not, see <http://www.gnu.org/licenses/>.
 
-********************************************************************
-
-$Revision: 6955 $:
-$Author: irascibl@gmail.com $:
-$Date: 2013-04-06 23:14:37 +0200 (Sa, 06. Apr 2013) $
-
 ********************************************************************/
 
 #include "modelpartshared.h"
@@ -54,9 +48,9 @@ void copyPinAttributes(QDomElement & from, QDomElement & to)
 ///////////////////////////////////////////////
 
 ViewImage::ViewImage(ViewLayer::ViewID vi) {
-    flipped = sticky = layers = 0;
-    canFlipVertical = canFlipHorizontal = false;
-    viewID = vi;
+	flipped = sticky = layers = 0;
+	canFlipVertical = canFlipHorizontal = false;
+	viewID = vi;
 }
 
 ///////////////////////////////////////////////
@@ -79,6 +73,8 @@ void ModelPartSharedRoot::setSearchTerm(const QString & searchTerm) {
 
 ///////////////////////////////////////////////
 
+const QString ModelPartShared::MNPropertyName = "mn";
+const QString ModelPartShared::MPNPropertyName = "mpn";
 const QString ModelPartShared::PartNumberPropertyName = "part number";
 
 ModelPartShared::ModelPartShared() {
@@ -93,30 +89,30 @@ ModelPartShared::ModelPartShared(QDomDocument & domDocument, const QString & pat
 
 	m_path = path;
 
-    setDomDocument(domDocument);
+	setDomDocument(domDocument);
 }
 
 void ModelPartShared::commonInit() {
 	m_moduleID = "";
-    m_dbid = 0;
-    m_ownerCount = 0;
-	m_hasZeroConnector = m_flippedSMD = m_connectorsInitialized = m_ignoreTerminalPoints = m_needsCopper1 = false;
-    m_superpart = NULL;
+	m_dbid = 0;
+	m_ownerCount = 0;
+	m_flippedSMD = m_connectorsInitialized = m_ignoreTerminalPoints = m_needsCopper1 = false;
+	m_superpart = nullptr;
 }
 
 ModelPartShared::~ModelPartShared() {
-	foreach (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
+	Q_FOREACH (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
 		delete connectorShared;
 	}
 	m_connectorSharedHash.clear();
 
-	foreach (ViewImage * viewImage, m_viewImages.values()) {
+	Q_FOREACH (ViewImage * viewImage, m_viewImages.values()) {
 		delete viewImage;
 	}
 	m_viewImages.clear();
 
 
-	foreach (BusShared * busShared, m_buses.values()) {
+	Q_FOREACH (BusShared * busShared, m_buses.values()) {
 		delete busShared;
 	}
 	m_buses.clear();
@@ -124,7 +120,7 @@ ModelPartShared::~ModelPartShared() {
 }
 
 bool ModelPartShared::setDomDocument(QDomDocument & domDocument) {
-    static qulonglong one = 1;
+	static qulonglong one = 1;
 
 	QDomElement root = domDocument.documentElement();
 	if (root.isNull()) {
@@ -148,59 +144,59 @@ bool ModelPartShared::setDomDocument(QDomDocument & domDocument) {
 		m_replacedby = version.attribute("replacedby");
 	}
 
-    QDomElement spice = root.firstChildElement("spice");
-    QDomElement line = spice.firstChildElement("line");
-    while (!line.isNull()) {
-        m_spice += line.text();
-        m_spice += "\n";
-        line = line.nextSiblingElement("line");
-    }
-    QDomElement model = spice.firstChildElement("model");
-    while (!model.isNull()) {
-        m_spiceModel += model.text();
-        m_spiceModel += "\n";
-        model = model.nextSiblingElement("model");
-    }
+	QDomElement spice = root.firstChildElement("spice");
+	QDomElement line = spice.firstChildElement("line");
+	while (!line.isNull()) {
+		m_spice += line.text();
+		m_spice += "\n";
+		line = line.nextSiblingElement("line");
+	}
+	QDomElement model = spice.firstChildElement("model");
+	while (!model.isNull()) {
+		m_spiceModel += model.text();
+		m_spiceModel += "\n";
+		model = model.nextSiblingElement("model");
+	}
 
 	populateTags(root, m_tags);
 	populateProperties(root, m_properties, m_displayKeys);
-    //foreach (QString key, m_displayKeys) {
-    //    DebugDialog::debug("set display " + m_moduleID + " " + key);
-    //}
+	//foreach (QString key, m_displayKeys) {
+	//    DebugDialog::debug("set display " + m_moduleID + " " + key);
+	//}
 	ensurePartNumberProperty();
 
 	m_moduleID = root.attribute("moduleId", "");
-    m_fritzingVersion = root.attribute("fritzingVersion", "");
+	m_fritzingVersion = root.attribute("fritzingVersion", "");
 
 	QDomElement views = root.firstChildElement("views");
 	if (!views.isNull()) {
 		QDomElement view = views.firstChildElement();
 		while (!view.isNull()) {
 			ViewLayer::ViewID viewID = ViewLayer::idFromXmlName(view.nodeName());
-            ViewImage * viewImage = new ViewImage(viewID);
-            m_viewImages.insert(viewID, viewImage);
-            viewImage->canFlipHorizontal = view.attribute("fliphorizontal","").compare("true") == 0;
-            viewImage->canFlipVertical = view.attribute("flipvertical","").compare("true") == 0;
+			auto * viewImage = new ViewImage(viewID);
+			m_viewImages.insert(viewID, viewImage);
+			viewImage->canFlipHorizontal = view.attribute("fliphorizontal","").compare("true") == 0;
+			viewImage->canFlipVertical = view.attribute("flipvertical","").compare("true") == 0;
 			QDomElement layers = view.firstChildElement("layers");
 			if (!layers.isNull()) {
-                viewImage->image = layers.attribute("image", "");
+				viewImage->image = layers.attribute("image", "");
 				QDomElement layer = layers.firstChildElement("layer");
 				while (!layer.isNull()) {
 					ViewLayer::ViewLayerID viewLayerID = ViewLayer::viewLayerIDFromXmlString(layer.attribute("layerId"));
-                    qulonglong sticky = (layer.attribute("sticky", "").compare("true") == 0) ? 1 : 0;
-                    viewImage->layers |= (one << viewLayerID); 
-                    viewImage->sticky |= (sticky << viewLayerID); 
+					qulonglong sticky = (layer.attribute("sticky", "").compare("true") == 0) ? 1 : 0;
+					viewImage->layers |= (one << viewLayerID);
+					viewImage->sticky |= (sticky << viewLayerID);
 					layer = layer.nextSiblingElement("layer");
 				}
 			}
 
-            addSchematicText(viewImage);
+			addSchematicText(viewImage);
 
 			view = view.nextSiblingElement();
 		}
 	}
 
-    return true;
+	return true;
 }
 
 void ModelPartShared::loadTagText(QDomElement parent, QString tagName, QString &field) {
@@ -380,14 +376,14 @@ const QList< QPointer<ConnectorShared> > ModelPartShared::connectorsShared() {
 }
 
 void ModelPartShared::setConnectorsShared(QList< QPointer<ConnectorShared> > connectors) {
-	for (int i = 0; i < connectors.size(); i++) {
-		ConnectorShared* cs = connectors[i];
+	for (auto & connector : connectors) {
+		ConnectorShared* cs = connector;
 		m_connectorSharedHash[cs->id()] = cs;
 	}
 }
 
 void ModelPartShared::setConnectorsInitialized(bool init) {
-    m_connectorsInitialized = init;
+	m_connectorsInitialized = init;
 }
 
 void ModelPartShared::initConnectors() {
@@ -395,11 +391,14 @@ void ModelPartShared::initConnectors() {
 		return;
 
 	QFile file(m_path);
+	if (!file.open(QIODevice::ReadOnly)) {
+		DebugDialog::debug(QString("Unable to open :%1").arg(m_path));
+	}
 	QString errorStr;
 	int errorLine;
 	int errorColumn;
 	QDomDocument doc;
-    doc.setContent(&file, &errorStr, &errorLine, &errorColumn);
+	doc.setContent(&file, &errorStr, &errorLine, &errorColumn);
 
 	m_connectorsInitialized = true;
 	QDomElement root = doc.documentElement();
@@ -416,18 +415,17 @@ void ModelPartShared::initConnectors() {
 	//DebugDialog::debug(QString("part:%1 %2").arg(m_moduleID).arg(m_title));
 	QDomElement connector = connectors.firstChildElement("connector");
 	while (!connector.isNull()) {
-		ConnectorShared * connectorShared = new ConnectorShared(connector);
+		auto * connectorShared = new ConnectorShared(connector);
 		m_connectorSharedHash.insert(connectorShared->id(), connectorShared);
 
 		connector = connector.nextSiblingElement("connector");
 	}
-    lookForZeroConnector();
 
 	QDomElement buses = root.firstChildElement("buses");
 	if (!buses.isNull()) {
 		QDomElement busElement = buses.firstChildElement("bus");
 		while (!busElement.isNull()) {
-			BusShared * busShared = new BusShared(busElement, m_connectorSharedHash);
+			auto * busShared = new BusShared(busElement, m_connectorSharedHash);
 			m_buses.insert(busShared->id(), busShared);
 
 			busElement = busElement.nextSiblingElement("bus");
@@ -464,25 +462,25 @@ void ModelPartShared::copy(ModelPartShared* other) {
 
 	setUri(other->uri());
 	setVersion(other->version());
-    setSuperpart(other->superpart());
+	setSuperpart(other->superpart());
 
-	foreach (ViewLayer::ViewID viewID, other->m_viewImages.keys()) {
-        ViewImage * otherViewImage = other->m_viewImages.value(viewID);
-        ViewImage * viewImage = new ViewImage(viewID);
-        viewImage->layers = otherViewImage->layers;
-        viewImage->sticky = otherViewImage->sticky;
-        viewImage->canFlipHorizontal = otherViewImage->canFlipHorizontal;
-        viewImage->canFlipVertical = otherViewImage->canFlipVertical;
-        viewImage->image = otherViewImage->image;
-        m_viewImages.insert(viewID, viewImage);
+	Q_FOREACH (ViewLayer::ViewID viewID, other->m_viewImages.keys()) {
+		ViewImage * otherViewImage = other->m_viewImages.value(viewID);
+		auto * viewImage = new ViewImage(viewID);
+		viewImage->layers = otherViewImage->layers;
+		viewImage->sticky = otherViewImage->sticky;
+		viewImage->canFlipHorizontal = otherViewImage->canFlipHorizontal;
+		viewImage->canFlipVertical = otherViewImage->canFlipVertical;
+		viewImage->image = otherViewImage->image;
+		m_viewImages.insert(viewID, viewImage);
 	}
 }
 
 void ModelPartShared::setProperty(const QString & key, const QString & value, bool showInLabel) {
 	m_properties.insert(key, value);
-    if (showInLabel) {
-        m_displayKeys.append(key);
-    }
+	if (showInLabel) {
+		m_displayKeys.append(key);
+	}
 }
 
 const QString & ModelPartShared::replacedby() {
@@ -506,9 +504,9 @@ bool ModelPartShared::needsCopper1() {
 }
 
 void ModelPartShared::connectorIDs(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID, QStringList & connectorIDs, QStringList & terminalIDs, QStringList & legIDs) {
-	foreach (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
+	Q_FOREACH (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
 		SvgIdLayer * svgIdLayer = connectorShared->fullPinInfo(viewID, viewLayerID);
-		if (svgIdLayer == NULL) {
+		if (svgIdLayer == nullptr) {
 			continue;
 		}
 		else {
@@ -520,101 +518,101 @@ void ModelPartShared::connectorIDs(ViewLayer::ViewID viewID, ViewLayer::ViewLaye
 }
 
 void ModelPartShared::flipSMDAnd() {
-    if (this->path().startsWith(ResourcePath)) {
-        // assume resources are set up exactly as intended
-        //DebugDialog::debug(QString("skip flip %1").arg(path()));
-        return;
-    }
-
-    static qulonglong one = 1;
-    ViewImage * viewImage = m_viewImages.value(ViewLayer::PCBView);
-
-    // needs to be called after initConnectors()
-    LayerList layerList = viewLayers(ViewLayer::PCBView);
-    if (layerList.isEmpty()) return;
-    if (!layerList.contains(ViewLayer::Copper0) && !layerList.contains(ViewLayer::Copper1)) {
-        return;
-    }
-
-    if (layerList.contains(ViewLayer::Copper0)) {
-        // THT here
-
-        if (!layerList.contains(ViewLayer::Copper1)) {
-            // fill in missing copper1 layer
-            m_needsCopper1 = true;
-            viewImage->layers |= (one << ViewLayer::Copper1);
-            copyPins(ViewLayer::Copper0, ViewLayer::Copper1);            
-        }
-
-        // prep for placing on the bottom
-	    if (layerList.contains(ViewLayer::Silkscreen1) && !layerList.contains(ViewLayer::Silkscreen0)) {
-            //DebugDialog::debug(QString("silk0 %1 %2").arg(this->title()).arg(this->moduleID()));
-            viewImage->layers |= (one << ViewLayer::Silkscreen0);
-            layerList << ViewLayer::Silkscreen0;
-	    }
-
-        if (layerList.contains(ViewLayer::Silkscreen0)) {
-	        viewImage->flipped |= (one << ViewLayer::Silkscreen0);
-        }
-
-        // swap layer for any tht part
-        if (!m_properties.keys().contains("layer")) {
-            // used for swapping part from copper1 to copper0
-            m_properties.insert("layer", "");
-        }
-
-        return;
-    }
-
-	setFlippedSMD(true);
-    // DebugDialog::debug("set flipped smd " + moduleID());
-    if (!m_properties.keys().contains("layer")) {
-        // used for swapping part from copper1 to copper0
-        m_properties.insert("layer", "");
-    }
-
-    if (!layerList.contains(ViewLayer::Copper0)) {
-        viewImage->layers |= (one << ViewLayer::Copper0);
-    }
-
-    viewImage->flipped |= (one << ViewLayer::Copper0);
-
-	if (layerList.contains(ViewLayer::Silkscreen1) && !layerList.contains(ViewLayer::Silkscreen0)) {
-        viewImage->layers |= (one << ViewLayer::Silkscreen0);
-        layerList << ViewLayer::Silkscreen0;
+	if (this->path().startsWith(ResourcePath)) {
+		// assume resources are set up exactly as intended
+		//DebugDialog::debug(QString("skip flip %1").arg(path()));
+		return;
 	}
 
-    if (layerList.contains(ViewLayer::Silkscreen0)) {
-	    viewImage->flipped |= (one << ViewLayer::Silkscreen0);
-    }
+	static qulonglong one = 1;
+	ViewImage * viewImage = m_viewImages.value(ViewLayer::PCBView);
 
-    //if (this->moduleID() == "df9d072afa2b594ac67b60b4153ff57b_29") {
-    //    DebugDialog::debug("alive in here");
-    //}
+	// needs to be called after initConnectors()
+	LayerList layerList = viewLayers(ViewLayer::PCBView);
+	if (layerList.isEmpty()) return;
+	if (!layerList.contains(ViewLayer::Copper0) && !layerList.contains(ViewLayer::Copper1)) {
+		return;
+	}
 
-    copyPins(ViewLayer::Copper1, ViewLayer::Copper0);   
+	if (layerList.contains(ViewLayer::Copper0)) {
+		// THT here
+
+		if (!layerList.contains(ViewLayer::Copper1)) {
+			// fill in missing copper1 layer
+			m_needsCopper1 = true;
+			viewImage->layers |= (one << ViewLayer::Copper1);
+			copyPins(ViewLayer::Copper0, ViewLayer::Copper1);
+		}
+
+		// prep for placing on the bottom
+		if (layerList.contains(ViewLayer::Silkscreen1) && !layerList.contains(ViewLayer::Silkscreen0)) {
+			//DebugDialog::debug(QString("silk0 %1 %2").arg(this->title()).arg(this->moduleID()));
+			viewImage->layers |= (one << ViewLayer::Silkscreen0);
+			layerList << ViewLayer::Silkscreen0;
+		}
+
+		if (layerList.contains(ViewLayer::Silkscreen0)) {
+			viewImage->flipped |= (one << ViewLayer::Silkscreen0);
+		}
+
+		// swap layer for any tht part
+		if (!m_properties.keys().contains("layer")) {
+			// used for swapping part from copper1 to copper0
+			m_properties.insert("layer", "");
+		}
+
+		return;
+	}
+
+	setFlippedSMD(true);
+	// DebugDialog::debug("set flipped smd " + moduleID());
+	if (!m_properties.keys().contains("layer")) {
+		// used for swapping part from copper1 to copper0
+		m_properties.insert("layer", "");
+	}
+
+	if (!layerList.contains(ViewLayer::Copper0)) {
+		viewImage->layers |= (one << ViewLayer::Copper0);
+	}
+
+	viewImage->flipped |= (one << ViewLayer::Copper0);
+
+	if (layerList.contains(ViewLayer::Silkscreen1) && !layerList.contains(ViewLayer::Silkscreen0)) {
+		viewImage->layers |= (one << ViewLayer::Silkscreen0);
+		layerList << ViewLayer::Silkscreen0;
+	}
+
+	if (layerList.contains(ViewLayer::Silkscreen0)) {
+		viewImage->flipped |= (one << ViewLayer::Silkscreen0);
+	}
+
+	//if (this->moduleID() == "df9d072afa2b594ac67b60b4153ff57b_29") {
+	//    DebugDialog::debug("alive in here");
+	//}
+
+	copyPins(ViewLayer::Copper1, ViewLayer::Copper0);
 }
 
 bool ModelPartShared::hasViewFor(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID, NULL);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID, NULL);
+	if (viewImage == nullptr) return false;
 
-    return viewImage->layers != 0;
+	return viewImage->layers != 0;
 }
 
 bool ModelPartShared::hasViewFor(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID) {
-    ViewImage * viewImage = m_viewImages.value(viewID, NULL);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID, NULL);
+	if (viewImage == nullptr) return false;
 
-    qulonglong one = 1;
-    return (viewImage->layers & (one << viewLayerID)) != 0;
+	qulonglong one = 1;
+	return (viewImage->layers & (one << viewLayerID)) != 0;
 }
 
 QString ModelPartShared::hasBaseNameFor(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID, NULL);
-    if (viewImage == NULL) return "";
+	ViewImage * viewImage = m_viewImages.value(viewID, NULL);
+	if (viewImage == nullptr) return "";
 
-    return viewImage->image;
+	return viewImage->image;
 }
 
 const QStringList & ModelPartShared::displayKeys() {
@@ -622,18 +620,20 @@ const QStringList & ModelPartShared::displayKeys() {
 }
 
 void ModelPartShared::ensurePartNumberProperty() {
-	if (!m_properties.keys().contains(PartNumberPropertyName)) {
-		m_properties.insert(PartNumberPropertyName, "");
-		m_displayKeys.append(PartNumberPropertyName);
+	for (auto&& propertyName : {MNPropertyName, MPNPropertyName, PartNumberPropertyName}) {
+		if (!m_properties.keys().contains(propertyName)) {
+			m_properties.insert(propertyName, "");
+			m_displayKeys.append(propertyName);
+		}
 	}
 }
 
 void ModelPartShared::setDBID(qulonglong dbid) {
-    m_dbid = dbid;
+	m_dbid = dbid;
 }
 
 qulonglong ModelPartShared::dbid() {
-    return m_dbid;
+	return m_dbid;
 }
 
 const QString & ModelPartShared::fritzingVersion() {
@@ -645,236 +645,220 @@ void ModelPartShared::setFritzingVersion(const QString & fv) {
 }
 
 void ModelPartShared::setViewImage(ViewImage * viewImage) {
-    ViewImage * old = m_viewImages.value(viewImage->viewID);
-    if (old) delete old;
-    m_viewImages.insert(viewImage->viewID, viewImage);
+	ViewImage * old = m_viewImages.value(viewImage->viewID);
+	if (old) delete old;
+	m_viewImages.insert(viewImage->viewID, viewImage);
 
-    addSchematicText(viewImage);
+	addSchematicText(viewImage);
 }
 
 const QList<ViewImage *> ModelPartShared::viewImages() {
-    return m_viewImages.values();
+	return m_viewImages.values();
 }
 
 QString ModelPartShared::imageFileName(ViewLayer::ViewID viewID, ViewLayer::ViewLayerID viewLayerID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return "";
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return "";
 
-    qulonglong one = 1;
-    if ((viewImage->layers & (one << viewLayerID)) == 0) return "";
+	qulonglong one = 1;
+	if ((viewImage->layers & (one << viewLayerID)) == 0) return "";
 
-    // DebugDialog::debug(QString("image filename %1 %2 %3").arg(viewImage->image).arg(viewID).arg(viewLayerID));
+	// DebugDialog::debug(QString("image filename %1 %2 %3").arg(viewImage->image).arg(viewID).arg(viewLayerID));
 
-    return viewImage->image;
+	return viewImage->image;
 }
 
 QString ModelPartShared::imageFileName(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return "";
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return "";
 
-    return viewImage->image;
+	return viewImage->image;
 }
 
 void ModelPartShared::setImageFileName(ViewLayer::ViewID viewID, const QString & filename) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return;
 
-    viewImage->image = filename;
+	viewImage->image = filename;
 }
 
 bool ModelPartShared::hasViewID(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return false;
 
-    return viewImage->layers != 0;
+	return viewImage->layers != 0;
 }
 
 bool ModelPartShared::hasMultipleLayers(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return false;
 
-    // http://eli.thegreenplace.net/2004/07/30/a-cool-algorithm-for-counting-ones-in-a-bitstring/
+	// http://eli.thegreenplace.net/2004/07/30/a-cool-algorithm-for-counting-ones-in-a-bitstring/
 
-    qulonglong layers = viewImage->layers;
-    return ((layers & (layers - 1)) != 0);
+	qulonglong layers = viewImage->layers;
+	return ((layers & (layers - 1)) != 0);
 }
 
 qulonglong layers(ViewImage * viewImage) {
-    return viewImage->layers;
+	return viewImage->layers;
 }
 
 qulonglong flipped(ViewImage * viewImage) {
-    return viewImage->flipped;
+	return viewImage->flipped;
 }
 
 LayerList ModelPartShared::viewLayersFlipped(ViewLayer::ViewID viewID) {
-    return viewLayersAux(viewID, flipped);
+	return viewLayersAux(viewID, flipped);
 }
 
 LayerList ModelPartShared::viewLayers(ViewLayer::ViewID viewID) {
-    return viewLayersAux(viewID, layers);
+	return viewLayersAux(viewID, layers);
 }
 
 LayerList ModelPartShared::viewLayersAux(ViewLayer::ViewID viewID, qulonglong (*accessor)(ViewImage *)) {
 
-    static QHash<qulonglong, ViewLayer::ViewLayerID> ToLayerIDs;
+	static QHash<qulonglong, ViewLayer::ViewLayerID> ToLayerIDs;
 
-    LayerList layerList;
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return layerList;
+	LayerList layerList;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return layerList;
 
-    // http://eli.thegreenplace.net/2004/07/30/a-cool-algorithm-for-counting-ones-in-a-bitstring/
+	// http://eli.thegreenplace.net/2004/07/30/a-cool-algorithm-for-counting-ones-in-a-bitstring/
 
-    if (ToLayerIDs.isEmpty()) {
-        qulonglong one = 1;
-        for (int ix = 0; ix < ViewLayer::ViewLayerCount; ix++) {
-            ToLayerIDs.insert(one, (ViewLayer::ViewLayerID) ix);
-            one = one << 1;
-        }
-    }
+	if (ToLayerIDs.isEmpty()) {
+		qulonglong one = 1;
+		for (int ix = 0; ix < ViewLayer::ViewLayerCount; ix++) {
+			ToLayerIDs.insert(one, (ViewLayer::ViewLayerID) ix);
+			one = one << 1;
+		}
+	}
 
-    qulonglong layers = accessor(viewImage);
-    while (layers) {
-        qulonglong removeLeast = layers & (layers - 1);
-        qulonglong diff = layers - removeLeast;
-        layerList << ToLayerIDs.value(diff);
-        layers = removeLeast;
-    }
+	qulonglong layers = accessor(viewImage);
+	while (layers) {
+		qulonglong removeLeast = layers & (layers - 1);
+		qulonglong diff = layers - removeLeast;
+		layerList << ToLayerIDs.value(diff);
+		layers = removeLeast;
+	}
 
-    return layerList;
+	return layerList;
 }
 
 
 bool ModelPartShared::canFlipHorizontal(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return false;
 
-    return viewImage->canFlipHorizontal;
+	return viewImage->canFlipHorizontal;
 }
 
 bool ModelPartShared::canFlipVertical(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return false;
 
-    return viewImage->canFlipVertical;
+	return viewImage->canFlipVertical;
 }
 
 bool ModelPartShared::anySticky(ViewLayer::ViewID viewID) {
-    ViewImage * viewImage = m_viewImages.value(viewID);
-    if (viewImage == NULL) return false;
+	ViewImage * viewImage = m_viewImages.value(viewID);
+	if (viewImage == nullptr) return false;
 
-    return (viewImage->sticky != 0);
+	return (viewImage->sticky != 0);
 }
 
 void ModelPartShared::addConnector(ConnectorShared * connectorShared)
 {
-    m_connectorSharedHash.insert(connectorShared->id(), connectorShared);
+	m_connectorSharedHash.insert(connectorShared->id(), connectorShared);
 }
 
 void ModelPartShared::copyPins(ViewLayer::ViewLayerID from, ViewLayer::ViewLayerID to) {
-    foreach (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
-        SvgIdLayer * svgIdLayer = connectorShared->fullPinInfo(ViewLayer::PCBView, to);
-        if (svgIdLayer) {
-            // already there
-            continue;
-        }
+	Q_FOREACH (ConnectorShared * connectorShared, m_connectorSharedHash.values()) {
+		SvgIdLayer * svgIdLayer = connectorShared->fullPinInfo(ViewLayer::PCBView, to);
+		if (svgIdLayer) {
+			// already there
+			continue;
+		}
 
-        svgIdLayer = connectorShared->fullPinInfo(ViewLayer::PCBView, from);
-        if (svgIdLayer == NULL) {
-            DebugDialog::debug(QString("missing connector in %1").arg(moduleID()));
-            continue;
-        }
+		svgIdLayer = connectorShared->fullPinInfo(ViewLayer::PCBView, from);
+		if (svgIdLayer == nullptr) {
+			DebugDialog::debug(QString("missing connector in %1").arg(moduleID()));
+			continue;
+		}
 
-        SvgIdLayer * newSvgIdLayer = svgIdLayer->copyLayer();
-        newSvgIdLayer->m_svgViewLayerID = to;
-        connectorShared->insertPin(ViewLayer::PCBView, newSvgIdLayer);
-    }
+		SvgIdLayer * newSvgIdLayer = svgIdLayer->copyLayer();
+		newSvgIdLayer->m_svgViewLayerID = to;
+		connectorShared->insertPin(ViewLayer::PCBView, newSvgIdLayer);
+	}
 }
 
 void ModelPartShared::insertBus(BusShared * busShared) {
-    m_buses.insert(busShared->id(), busShared);
-}
-
-void ModelPartShared::lookForZeroConnector() {
-    foreach (QString key, m_connectorSharedHash.keys()) {
-        int ix = IntegerFinder.indexIn(key);
-        if (ix >= 0) {
-            if (IntegerFinder.cap(0) == "0") {
-                m_hasZeroConnector = true;
-                return;
-            }
-        }
-    }
-}
-
-bool ModelPartShared::hasZeroConnector() {
-    return m_hasZeroConnector;
+	m_buses.insert(busShared->id(), busShared);
 }
 
 void ModelPartShared::addOwner(QObject * owner) {
-    m_ownerCount++;
-    connect(owner, SIGNAL(destroyed()), this, SLOT(removeOwner()));
+	m_ownerCount++;
+	connect(owner, SIGNAL(destroyed()), this, SLOT(removeOwner()));
 }
 
 void ModelPartShared::removeOwner() {
-    if (--m_ownerCount == 0) {
-        // DebugDialog::debug(QString("last owner %1").arg(moduleID()));
-        // this->deleteLater();
-    }
+	if (--m_ownerCount == 0) {
+		// DebugDialog::debug(QString("last owner %1").arg(moduleID()));
+		// this->deleteLater();
+	}
 }
 
 void ModelPartShared::addSchematicText(ViewImage * viewImage) {
-    if (viewImage->viewID != ViewLayer::SchematicView) return;
+	if (viewImage->viewID != ViewLayer::SchematicView) return;
 
-    qulonglong one = 1;
-    if ((viewImage->layers & (one << ViewLayer::Schematic)) == 0) return;
+	qulonglong one = 1;
+	if ((viewImage->layers & (one << ViewLayer::Schematic)) == 0) return;
 
-    viewImage->canFlipHorizontal = viewImage->canFlipVertical = true;
-    viewImage->layers |= (one << ViewLayer::SchematicText); 
+	viewImage->canFlipHorizontal = viewImage->canFlipVertical = true;
+	viewImage->layers |= (one << ViewLayer::SchematicText);
 }
 
 bool ModelPartShared::showInLabel(const QString & propertyName) {
-    //foreach (QString key, m_displayKeys) {
-    //    DebugDialog::debug("check display " + m_moduleID + " " + key);
-    //}
-    return m_displayKeys.contains(propertyName, Qt::CaseInsensitive);
+	//foreach (QString key, m_displayKeys) {
+	//    DebugDialog::debug("check display " + m_moduleID + " " + key);
+	//}
+	return m_displayKeys.contains(propertyName, Qt::CaseInsensitive);
 }
 
 const QList< QPointer<ModelPartShared> > & ModelPartShared::subparts() {
-    return m_subparts;
+	return m_subparts;
 }
 
 void ModelPartShared::addSubpart(ModelPartShared * subpart) {
-    if (subpart == NULL) return;
+	if (subpart == nullptr) return;
 
-    m_subparts.append(subpart);
-    subpart->setSuperpart(this);
+	m_subparts.append(subpart);
+	subpart->setSuperpart(this);
 }
 
 ModelPartShared * ModelPartShared::superpart() {
-    return m_superpart;
+	return m_superpart;
 }
 
 void ModelPartShared::setSuperpart(ModelPartShared * sup) {
-    m_superpart = sup;
+	m_superpart = sup;
 }
 
 bool ModelPartShared::hasSubparts() {
-    return m_subparts.count() > 0;
+	return m_subparts.count() > 0;
 }
 
 void ModelPartShared::setSubpartID(const QString & id) {
-    m_subpartID = id;
+	m_subpartID = id;
 }
 
 const QString & ModelPartShared::subpartID() const {
-    return m_subpartID;
+	return m_subpartID;
 }
 
 QPointF ModelPartShared::subpartOffset() const {
-    return m_subpartOffset;
+	return m_subpartOffset;
 }
 
 void ModelPartShared::setSubpartOffset(QPointF p) {
-    m_subpartOffset = p;
+	m_subpartOffset = p;
 }
